import re
import unittest

from getgauge.registry import Registry


class RegistryTests(unittest.TestCase):
    def setUp(self):
        global registry
        registry = Registry()

    def test_Registry_add_step_definition(self):
        infos = [{'text': 'Say <hello> to <getgauge>', 'func': 'func'},
                 {'text': 'Step 1', 'func': 'func1'}]

        for info in infos:
            registry.add_step(info['text'], info['func'], '')

        registry.steps().sort()

        info_text = [info['text'] for info in infos]
        info_text.sort()

        self.assertEqual(info_text, registry.steps())

        for info in infos:
            parsed_step_text = re.sub('<[^<]+?>', '{}', info['text'])
            self.assertEqual(info['func'],
                             registry.get_info_for(parsed_step_text).impl)

    def test_Registry_add_step_definition_with_continue_on_failure(self):
        registry.add_step('Step 1', 'func', '')
        registry.continue_on_failure('func', [RuntimeError])

        self.assertTrue(registry.is_continue_on_failure('func', RuntimeError()))

    def test_Registry_add_step_definition_with_continue_on_failure_for_different_exceptions(self):
        registry.add_step('Step 1', 'func', '')
        registry.continue_on_failure('func', [RuntimeError])

        self.assertFalse(registry.is_continue_on_failure('func', IndexError()))

    def test_Registry_add_step_definition_with_parent_class_continue_on_failure(self):
        registry.add_step('Step 1', 'func', '')
        registry.continue_on_failure('func', [Exception])

        self.assertTrue(registry.is_continue_on_failure('func', RuntimeError()))

    def test_Registry_add_step_definition_with_alias(self):
        registry.add_step(['Say <hello> to <getgauge>.',
                           'Tell <hello> to <getgauge>.'], 'impl', '')

        info1 = registry.get_info_for('Say {} to {}.')
        info2 = registry.get_info_for('Tell {} to {}.')

        self.assertTrue(info1.has_alias)
        self.assertTrue(info2.has_alias)

    def test_Registry_get_step_info(self):
        infos = [{'text': 'Say <hello> to <getgauge>', 'func': 'func'},
                 {'text': 'Step 1', 'func': 'func1'}]

        for info in infos:
            registry.add_step(info['text'], info['func'], '')

        self.assertEqual('Say <hello> to <getgauge>',
                         registry.get_info_for('Say {} to {}').step_text)

        self.assertEqual('Say {} to {}',
                         registry.get_info_for('Say {} to {}').parsed_step_text)

        self.assertEqual('Step 1', registry.get_info_for('Step 1').step_text)

        self.assertIsNone(registry.get_info_for('Step21').step_text)

    def test_Registry_is_step_implemented(self):
        infos = [{'text': 'Say <hello> to <getgauge>', 'func': 'func'},
                 {'text': 'Step 1', 'func': 'func1'}]

        for info in infos:
            registry.add_step(info['text'], info['func'], '')

        for info in infos:
            parsed_step_text = re.sub('<[^<]+?>', '{}', info['text'])
            self.assertTrue(registry.is_implemented(parsed_step_text))

    def test_Registry_has_multiple_impls(self):
        infos = [{'text': 'Say <hello> to <getgauge>', 'func': 'func'},
                 {'text': 'Say <hello> to <getgauge>', 'func': 'func'},
                 {'text': 'Step 1', 'func': 'func1'}]
        for info in infos:
            registry.add_step(info['text'], info['func'], '')

        parsed_step_text = re.sub('<[^<]+?>', '{}', infos[0]['text'])

        self.assertTrue(registry.has_multiple_impls(parsed_step_text))
        self.assertFalse(registry.has_multiple_impls(infos[2]['text']))

    def test_Registry_get_multiple_impls(self):
        infos = [{'text': 'Say <hello> to <getgauge>',
                  'func': 'func', 'line': 1},
                 {'text': 'Say <hello> to <getgauge>',
                  'func': 'func', 'line': 2},
                 {'text': 'Step 1', 'func': 'func1', 'line': 3}]

        for info in infos:
            registry.add_step(info['text'], info['func'],
                              '', {'start': info['line']})

        parsed_step_text = re.sub('<[^<]+?>', '{}', infos[0]['text'])

        self.assertTrue(registry.has_multiple_impls(parsed_step_text))
        self.assertEqual(
            set([info.span['start']
                 for info in registry.get_infos_for(parsed_step_text)]),
            {infos[0]['line'], infos[1]['line']})

        self.assertEqual(
            set([info.step_text
                 for info in registry.get_infos_for(parsed_step_text)]),
            {infos[0]['text'], infos[1]['text']})

    def test_Registry_before_suite(self):
        infos = ['before suite func', 'before suite func1']
        for info in infos:
            registry.add_before_suite(info)

        self.assertEqual(infos, [i.impl for i in registry.before_suite()])

    def test_Registry_after_suite(self):
        infos = ['after suite func', 'after suite func1']
        for info in infos:
            registry.add_after_suite(info)

        self.assertEqual(infos, [i.impl for i in registry.after_suite()])

    def test_Registry_before_spec(self):
        infos = ['before spec func', 'before spec func1']
        for info in infos:
            registry.add_before_spec(info)

        self.assertEqual(infos, [i.impl for i in registry.before_spec()])

    def test_Registry_after_spec(self):
        infos = ['after spec func', 'after spec func1']
        for info in infos:
            registry.add_after_spec(info)

        self.assertEqual(infos, [i.impl for i in registry.after_spec()])

    def test_Registry_before_spec_with_tags(self):
        info1 = {'tags': None, 'func': 'before spec func'}
        info2 = {'tags': '<A> and <b> and not <c>',
                 'func': 'before spec func1'}
        info3 = {'tags': '<A> and (<b> or not <c>)',
                 'func': 'before spec func2'}
        infos = [info1, info2, info3]

        for info in infos:
            registry.add_before_spec(info['func'], info['tags'])

        self.assertEqual([info1['func']], [i.impl for i in registry.before_spec([])])
        self.assertEqual([x['func'] for x in infos], [i.impl for i in registry.before_spec(['A', 'b'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.before_spec(['A', 'b', 'c'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.before_spec(['A'])])
        self.assertEqual([info1['func']], [i.impl for i in registry.before_spec(['A', 'c'])])

    def test_Registry_after_spec_with_tags(self):
        info1 = {'tags': None, 'func': 'after spec func'}
        info2 = {'tags': '<A> and <b> and not <c>', 'func': 'after spec func1'}
        info3 = {'tags': '<A> and (<b> or not <c>)',
                 'func': 'after spec func2'}
        infos = [info1, info2, info3]

        for info in infos:
            registry.add_after_spec(info['func'], info['tags'])

        self.assertEqual([info1['func']], [i.impl for i in registry.after_spec([])])
        self.assertEqual([x['func'] for x in infos], [i.impl for i in registry.after_spec(['A', 'b'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.after_spec(['A', 'b', 'c'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.after_spec(['A'])])
        self.assertEqual([info1['func']], [i.impl for i in registry.after_spec(['A', 'c'])])

    def test_Registry_before_scenario(self):
        infos = ['before scenario func', 'before scenario func1']
        for info in infos:
            registry.add_before_scenario(info)

        self.assertEqual(infos, [i.impl for i in registry.before_scenario()])

    def test_Registry_after_scenario(self):
        infos = ['after scenario func', 'after scenario func1']
        for info in infos:
            registry.add_after_scenario(info)

        self.assertEqual(infos, [i.impl for i in registry.after_scenario()])

    def test_Registry_before_scenario_with_tags(self):
        info1 = {'tags': None, 'func': 'before scenario func'}
        info2 = {'tags': '<A> and <b> and not <c>',
                 'func': 'before scenario func1'}
        info3 = {'tags': '<A> and (<b> or not <c>)',
                 'func': 'before scenario func2'}
        infos = [info1, info2, info3]

        for info in infos:
            registry.add_before_scenario(info['func'], info['tags'])

        self.assertEqual([info1['func']],
                         [i.impl for i in registry.before_scenario([])])

        self.assertEqual([x['func'] for x in infos],
                         [i.impl for i in registry.before_scenario(['A', 'b'])])

        self.assertEqual([info1['func'], info3['func']],
                         [i.impl for i in registry.before_scenario(['A', 'b', 'c'])])

        self.assertEqual([info1['func'], info3['func']],
                         [i.impl for i in registry.before_scenario(['A'])])

        self.assertEqual([info1['func']], [i.impl for i in registry.before_scenario(['A', 'c'])])

    def test_Registry_after_scenario_with_tags(self):
        info1 = {'tags': None, 'func': 'after scenario func'}
        info2 = {'tags': '<A> and <b> and not <c>',
                 'func': 'after scenario func1'}
        info3 = {'tags': '<A> and (<b> or not <c>)',
                 'func': 'after scenario func2'}
        infos = [info1, info2, info3]

        for info in infos:
            registry.add_after_scenario(info['func'], info['tags'])

        self.assertEqual([info1['func']], [i.impl for i in registry.after_scenario([])])
        self.assertEqual([x['func'] for x in infos],
                         [i.impl for i in registry.after_scenario(['A', 'b'])])

        self.assertEqual([info1['func'], info3['func']],
                         [i.impl for i in registry.after_scenario(['A', 'b', 'c'])])

        self.assertEqual([info1['func'], info3['func']],
                         [i.impl for i in registry.after_scenario(['A'])])

        self.assertEqual([info1['func']], [i.impl for i in registry.after_scenario(['A', 'c'])])

    def test_Registry_before_step(self):
        infos = ['before step func', 'before step func1']
        for info in infos:
            registry.add_before_step(info)

        self.assertEqual(infos, [i.impl for i in registry.before_step()])

    def test_Registry_after_step(self):
        infos = ['after step func', 'after step func1']
        for info in infos:
            registry.add_after_step(info)

        self.assertEqual(infos, [i.impl for i in registry.after_step()])

    def test_Registry_before_step_with_tags(self):
        info1 = {'tags': None, 'func': 'before step func'}
        info2 = {'tags': '<A> and <b> and not <c>',
                 'func': 'before step func1'}
        info3 = {'tags': '<A> and (<b> or not <c>)',
                 'func': 'before step func2'}
        infos = [info1, info2, info3]

        for info in infos:
            registry.add_before_step(info['func'], info['tags'])

        self.assertEqual([info1['func']], [i.impl for i in registry.before_step([])])
        self.assertEqual([x['func'] for x in infos], [i.impl for i in registry.before_step(['A', 'b'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.before_step(['A', 'b', 'c'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.before_step(['A'])])
        self.assertEqual([info1['func']], [i.impl for i in registry.before_step(['A', 'c'])])

    def test_Registry_after_step_with_tags(self):
        info1 = {'tags': None, 'func': 'after step func'}
        info2 = {'tags': '<A> and <b> and not <c>',
                 'func': 'after step func1'}
        info3 = {'tags': '<A> and (<b> or not <c>)',
                 'func': 'after step func2'}
        infos = [info1, info2, info3]

        for info in infos:
            registry.add_after_step(info['func'], info['tags'])

        self.assertEqual([info1['func']], [i.impl for i in registry.after_step([])])
        self.assertEqual([x['func'] for x in infos], [i.impl for i in registry.after_step(['A', 'b'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.after_step(['A', 'b', 'c'])])
        self.assertEqual([info1['func'], info3['func']], [i.impl for i in registry.after_step(['A'])])
        self.assertEqual([info1['func']], [i.impl for i in registry.after_step(['A', 'c'])])

    def test_Registry__step_positions_of_a_given_file(self):
        infos = [{'text': 'Say <hello> to <getgauge>', 'func': 'func', 'file_name': 'foo.py', 'span': {'start': 1}},
                 {'text': 'Step 1', 'func': 'func1', 'file_name': 'bar.py', 'span': {'start': 3}}]

        for info in infos:
            registry.add_step(info['text'], info['func'], info['file_name'], info['span'])

        positions = registry.get_step_positions('foo.py')

        self.assertIn({'stepValue': 'Say {} to {}', 'span': {'start': 1}}, positions)
        self.assertNotIn({'stepValue': 'Step 1', 'span': {'start': 3}}, positions)

        positions = registry.get_step_positions('bar.py')

        self.assertIn({'stepValue': 'Step 1', 'span': {'start': 3}}, positions)

    def test_Registry_remove_steps_of_a_given_file(self):
        infos = [{'text': 'Say <hello> to <getgauge>', 'func': 'func', 'file_name': 'foo.py'},
                 {'text': 'Step 1', 'func': 'func1', 'file_name': 'bar.py'}]

        for info in infos:
            registry.add_step(info['text'], info['func'], info['file_name'])

        registry.remove_steps('foo.py')

        self.assertFalse(registry.is_implemented('Say {} to {}'))
        self.assertTrue(registry.is_implemented('Step 1'))

    def test_Registry_remove_steps_of_a_given_file_with_duplicate_implementations(self):
        infos = [{'text': 'Step 1', 'func': 'func', 'file_name': 'foo.py'},
                 {'text': 'Step 1', 'func': 'func1', 'file_name': 'bar.py'}]

        for info in infos:
            registry.add_step(info['text'], info['func'], info['file_name'])

        self.assertTrue(registry.has_multiple_impls('Step 1'))

        registry.remove_steps('foo.py')

        self.assertTrue(registry.is_implemented('Step 1'))
        self.assertFalse(registry.has_multiple_impls('Step 1'))

    def test_Registry_is_file_cached(self):
        info = {'text': 'Say <hello> to <getgauge>', 'func': 'func', 'file_name': 'foo.py'}
        registry.add_step(info['text'], info['func'], info['file_name'])

        self.assertTrue(registry.is_file_cached(info['file_name']))

    def test_Registry_add_step_with_empty_arg(self):
        info = {'text': 'Step <>', 'func': 'func', 'file_name': 'foo.py'}

        registry.add_step(info['text'], info['func'], info['file_name'])

        self.assertTrue(registry.is_implemented('Step {}'))

    def test_Registry_get_all_methods_in_should_give_all_the_methods_define_in_that_file(self):
        hooks_infos = [{'func': 'func1', 'file_name': 'foo.py'},
                       {'func': 'func2', 'file_name': 'foo.py'},
                       {'func': 'func3', 'file_name': 'bar.py'}
                       ]
        for info in hooks_infos:
            registry.add_before_step(info['func'], "", info['file_name'])
        step_infos = [{'text': 'Step 1', 'func': 'func', 'file_name': 'foo.py'},
                      {'text': 'Step 1', 'func': 'func1', 'file_name': 'bar.py'}]
        for info in step_infos:
            registry.add_step(info['text'], info['func'], info['file_name'])

        self.assertEqual(3, len(registry.get_all_methods_in("foo.py")))
        self.assertEqual(2, len(registry.get_all_methods_in("bar.py")))

    def tearDown(self):
        global registry
        registry = Registry()


if __name__ == '__main__':
    unittest.main()



import os
import tempfile
from unittest import TestCase, main
from uuid import uuid1

from getgauge.messages.messages_pb2 import Message
from getgauge.python import (DataStore, DataStoreContainer, DictObject,
                             ExecutionContext, Messages, Scenario,
                             Specification, Step, Table,
                             create_execution_context_from, data_store)
from getgauge.registry import MessagesStore, ScreenshotsStore, registry

try:
    from collections.abc import MutableMapping
except ImportError:
    from collections import MutableMapping

registry.clear()


class MessagesTests(TestCase):
    def test_pending_messages(self):
        messages = ['HAHAHAH', 'HAHAHAH1', 'HAHAHAH2', 'HAHAHAH3']
        for message in messages:
            Messages.write_message(message)
        pending_messages = MessagesStore.pending_messages()
        self.assertEqual(messages, pending_messages)

    def test_clear(self):
        messages = ['HAHAHAH', 'HAHAHAH1', 'HAHAHAH2', 'HAHAHAH3']
        for message in messages:
            Messages.write_message(message)
        MessagesStore.clear()
        pending_messages = MessagesStore.pending_messages()
        self.assertEqual([], pending_messages)

    def test_pending_messages_gives_only_those_messages_which_are_not_reported(self):
        messages = ['HAHAHAH', 'HAHAHAH1', 'HAHAHAH2', 'HAHAHAH3']
        for message in messages:
            Messages.write_message(message)
        pending_messages = MessagesStore.pending_messages()
        self.assertEqual(messages, pending_messages)

        pending_messages = MessagesStore.pending_messages()

        self.assertEqual([], pending_messages)

        messages = ['HAHAHAH', 'HAHAHAH1']
        for message in messages:
            Messages.write_message(message)

        pending_messages = MessagesStore.pending_messages()

        self.assertEqual(messages, pending_messages)


class DataStoreTests(TestCase):
    def test_data_store(self):
        store = DataStore()
        values = {'key': 'HAHAHAH',
                  'key1': 'HAHAHAH1',
                  'key2': 'HAHAHAH2',
                  'key3': 'HAHAHAH3'}

        for value in values:
            store.put(value, value)

        for key, value in values.items():
            store.put(key, value)

        for key, value in values.items():
            self.assertEqual(store.get(key), value)

    def test_data_store_clear(self):
        store = DataStore()
        values = {'key': 'HAHAHAH',
                  'key1': 'HAHAHAH1',
                  'key2': 'HAHAHAH2',
                  'key3': 'HAHAHAH3'}

        for value in values:
            store.put(value, value)

        for key, value in values.items():
            store.put(key, value)

        for value in values:
            self.assertTrue(store.is_present(value))

        store.clear()

        for value in values:
            self.assertFalse(store.is_present(value))

    def test_data_store_equality(self):
        store = DataStore()
        store1 = DataStore()

        self.assertEqual(store, store1)

        store.put('a', 'b')
        store1.put('a', 'b')

        self.assertEqual(store, store1)

        store.put('b', 'b')
        store1.put('c', 'b')

        self.assertNotEqual(store, store1)

    def test_data_store_as_proxy(self):
        store = {}
        proxy = DataStore(store)

        store['a'] = 'alpha'
        self.assertTrue(proxy.is_present('a'))
        self.assertEqual(proxy.get('a'), store['a'])

        proxy.put('b', 'beta')
        self.assertIn('b', store)
        self.assertEqual(store['b'], proxy.get('b'))

        proxy2 = DataStore(store)
        self.assertEqual(proxy, proxy2)

        proxy.clear()
        self.assertDictEqual(store, {})
        self.assertFalse(proxy2.is_present('a'))
        self.assertFalse(proxy2.is_present('b'))

        proxy2.put('c', 'charlie')
        proxy2.put('d', 'delta')
        self.assertIn('c', store)
        self.assertIn('d', store)
        self.assertTrue(proxy.is_present('c'))
        self.assertTrue(proxy.is_present('d'))

        store.clear()
        self.assertFalse(proxy.is_present('c'))
        self.assertFalse(proxy.is_present('d'))
        self.assertFalse(proxy2.is_present('c'))
        self.assertFalse(proxy2.is_present('d'))


class DictObjectTests(TestCase):
    def test_attribute_access(self):
        store = DictObject()

        store['a'] = 'alpha'
        self.assertEqual(store.a, 'alpha')

        store.clear()
        with self.assertRaises(AttributeError):
            store.a

        store.b = 'beta'
        self.assertIn('b', store)
        del store.b
        self.assertNotIn('b', store)

        store['k e y'] = 'value'
        with self.assertRaises(AttributeError):
            store.k


class DataStoreContainerTests(TestCase):
    def test_data_store_container(self):
        store = DataStoreContainer()
        # Verify that each store is a mapping
        self.assertIsInstance(store.scenario, MutableMapping)
        self.assertIsInstance(store.spec, MutableMapping)
        self.assertIsInstance(store.suite, MutableMapping)
        # Verify that stores can not be reassigned
        with self.assertRaises(AttributeError):
            store.scenario = {}
        with self.assertRaises(AttributeError):
            store.spec = {}
        with self.assertRaises(AttributeError):
            store.suite = {}

    def test_module_data_store(self):
        self.assertIsInstance(data_store, DataStoreContainer)


class ProtoTable:
    def __init__(self, table_dict):
        self.headers = ProtoRow(table_dict['headers']['cells'])
        self.rows = [ProtoRow(row['cells']) for row in table_dict['rows']]


class ProtoRow:
    def __init__(self, cells):
        self.cells = cells


class TableTests(TestCase):
    def test_Table(self):
        headers = ['Product', 'Description']
        rows = [{'cells': ['Gauge', 'Test automation with ease']},
                {'cells': ['Mingle', 'Agile project management']},
                {'cells': ['Snap', 'Hosted continuous integration']},
                {'cells': ['Gocd', 'Continuous delivery platform']}]

        proto_table = ProtoTable({'headers': {'cells': headers}, 'rows': rows})

        table = Table(proto_table)

        expected_rows = [row['cells'] for row in rows]
        expected_column_1 = [row['cells'][0] for row in rows]
        expected_column_2 = [row['cells'][1] for row in rows]

        self.assertEqual(headers, table.headers)
        self.assertEqual(expected_rows, table.rows)
        self.assertEqual(expected_column_1,
                         table.get_column_values_with_index(1))
        self.assertEqual(expected_column_2,
                         table.get_column_values_with_index(2))
        self.assertEqual(expected_column_1,
                         table.get_column_values_with_name(headers[0]))
        self.assertEqual(expected_column_2,
                         table.get_column_values_with_name(headers[1]))

        for row in expected_rows:
            self.assertEqual(row, table.get_row(expected_rows.index(row) + 1))

        with self.assertRaises(IndexError):
            table.get_row(5)

    def test_Table_with_index_access(self):
        headers = ['Product', 'Description']
        rows = [{'cells': ['Gauge', 'Test automation with ease']},
                {'cells': ['Mingle', 'Agile project management']},
                {'cells': ['Snap', 'Hosted continuous integration']},
                {'cells': ['Gocd', 'Continuous delivery platform']}]

        proto_table = ProtoTable({'headers': {'cells': headers}, 'rows': rows})

        table = Table(proto_table)

        expected_rows = [row['cells'] for row in rows]

        for row in expected_rows:
            self.assertEqual(row, table[expected_rows.index(row)])

        for row in table:
            self.assertIn(row, expected_rows)

        with self.assertRaises(IndexError):
            table.get_row(5)

    def test_Table_equality(self):
        headers = ['Product', 'Description']
        rows = [{'cells': ['Gauge', 'Test automation with ease']},
                {'cells': ['Mingle', 'Agile project management']},
                {'cells': ['Snap', 'Hosted continuous integration']},
                {'cells': ['Gocd', 'Continuous delivery platform']}]

        proto_table = ProtoTable({'headers': {'cells': headers}, 'rows': rows})

        table = Table(proto_table)
        table1 = Table(proto_table)

        self.assertEqual(table, table1)

    def test_Table__str__(self):
        headers = ['Word', 'Vowel Count']
        rows = [{'cells': ['Gauge', '3']},
                {'cells': ['Mingle', '2']},
                {'cells': ['Snap', '1']},
                {'cells': ['GoCD', '1']},
                {'cells': ['Rhythm', '0']}]

        proto_table = ProtoTable({'headers': {'cells': headers}, 'rows': rows})

        table = str(Table(proto_table))

        self.assertEqual(table, """|Word  |Vowel Count|
|------|-----------|
|Gauge |3          |
|Mingle|2          |
|Snap  |1          |
|GoCD  |1          |
|Rhythm|0          |""")

    def test_Table__str__without_rows(self):
        headers = ['Word', 'Vowel Count']
        rows = []

        proto_table = ProtoTable({'headers': {'cells': headers}, 'rows': rows})

        table = str(Table(proto_table))

        self.assertEqual(table, """|Word|Vowel Count|
|----|-----------|""")


class SpecificationTests(TestCase):
    def test_Specification(self):
        name = 'NAME'
        file_name = 'FILE_NAME'
        tags = ['TAGS']
        specification = Specification(name, file_name, False, tags)

        self.assertEqual(specification.name, name)
        self.assertEqual(specification.file_name, file_name)
        self.assertFalse(specification.is_failing)
        self.assertEqual(specification.tags, tags)

    def test_Specification_equality(self):
        name = 'NAME'
        file_name = 'FILE_NAME'
        tags = ['TAGS']
        specification = Specification(name, file_name, False, tags)
        specification1 = Specification(name, file_name, False, tags)

        self.assertEqual(specification, specification1)


class ScenarioTests(TestCase):
    def test_Scenario(self):
        name = 'NAME3'
        tags = ['TAGS']
        scenario = Scenario(name, False, tags)

        self.assertEqual(scenario.name, name)
        self.assertFalse(scenario.is_failing)
        self.assertEqual(scenario.tags, tags)

    def test_Scenario_equality(self):
        name = 'NAME2'
        tags = ['TAGS']
        scenario = Scenario(name, False, tags)
        scenario1 = Scenario(name, False, tags)

        self.assertEqual(scenario, scenario1)


class StepTests(TestCase):
    def test_Step(self):
        name = 'NAME1'
        step = Step(name, False)

        self.assertEqual(step.text, name)
        self.assertFalse(step.is_failing)

    def test_Step_equality(self):
        name = 'NAME1'
        step = Step(name, False)
        step1 = Step(name, False)

        self.assertEqual(step, step1)


class ExecutionContextTests(TestCase):
    def test_ExecutionContextTests(self):
        name = 'NAME'
        file_name = 'FILE_NAME'
        tags = ['TAGS']
        specification = Specification(name, file_name, False, tags)
        scenario = Scenario(name, False, tags)
        step = Step(name, False)

        context = ExecutionContext(specification, scenario, step)
        self.assertEqual(specification, context.specification)
        self.assertEqual(scenario, context.scenario)
        self.assertEqual(step, context.step)

    def test_ExecutionContextTests_equality(self):
        name = 'NAME'
        file_name = 'FILE_NAME'
        tags = ['TAGS']
        specification = Specification(name, file_name, False, tags)
        scenario = Scenario(name, False, tags)
        step = Step(name, False)

        context = ExecutionContext(specification, scenario, step)
        context1 = ExecutionContext(specification, scenario, step)

        self.assertEqual(context, context1)

    def test_create_execution_context_from(self):
        message = Message()
        spec_name = 'SPEC_NAME'
        spec_file_name = 'SPEC_FILE_NAME'
        scenario_name = 'SCENARIO_NAME'
        step_name = 'STEP_NAME'

        message.executionStartingRequest.\
            currentExecutionInfo.currentSpec.name = spec_name
        message.executionStartingRequest.\
            currentExecutionInfo.currentSpec.fileName = spec_file_name
        message.executionStartingRequest.\
            currentExecutionInfo.currentSpec.isFailed = True
        message.executionStartingRequest.\
            currentExecutionInfo.currentScenario.name = scenario_name
        message.executionStartingRequest.\
            currentExecutionInfo.currentScenario.isFailed = False
        message.executionStartingRequest.\
            currentExecutionInfo.currentStep.step.actualStepText = step_name
        message.executionStartingRequest.\
            currentExecutionInfo.currentStep.isFailed = True
        message.executionStartingRequest. \
            currentExecutionInfo.currentStep.errorMessage = "Error"
        message.executionStartingRequest. \
            currentExecutionInfo.currentStep.stackTrace = "stack trace"

        specification = Specification(spec_name, spec_file_name, True, [])
        scenario = Scenario(scenario_name, False, [])
        step = Step(step_name, True, "Error", "stack trace")

        context = ExecutionContext(specification, scenario, step)

        expected_execution_context = create_execution_context_from(
            message.executionStartingRequest.currentExecutionInfo)
        self.assertEqual(expected_execution_context, context)


class DecoratorTests(TestCase):
    def setUp(self):
        from tests.test_data import impl_stubs
        impl_stubs.step1()

    def test_step_decorator(self):
        steps = registry.steps()
        expected = {'Step 1', 'Step 2'}
        self.assertEqual(expected, set(steps))

    def test_continue_on_failure(self):
        step1 = registry.get_info_for('Step 1').impl
        step2 = registry.get_info_for('Step 2').impl

        self.assertFalse(
            registry.is_continue_on_failure(step1, RuntimeError()))
        self.assertTrue(
            registry.is_continue_on_failure(step2, RuntimeError()))

    def test_before_step_decorator(self):
        funcs = registry.before_step()
        expected = ['before_step1', 'before_step2']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_after_step_decorator(self):
        funcs = registry.after_step()
        expected = ['after_step1']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_before_scenario_decorator(self):
        funcs = registry.before_scenario(['haha', 'hehe'])
        expected = ['before_scenario1', 'before_scenario2']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_after_scenario_decorator(self):
        funcs = registry.after_scenario(['haha', 'hehe'])
        expected = ['after_scenario1', 'after_scenario2']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_before_spec_decorator(self):
        funcs = registry.before_spec(['haha', 'hehe'])
        expected = ['before_spec1', 'before_spec2']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_after_spec_decorator(self):
        funcs = registry.after_spec(['haha', 'hehe'])
        expected = ['after_spec1', 'after_spec2']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_before_suite_decorator(self):
        funcs = registry.before_suite()
        expected = ['before_suite1']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_after_suite_decorator(self):
        funcs = registry.after_suite()
        expected = ['after_suite1', 'after_suite2']
        self.assertEqual(expected, [info.impl.__name__ for info in funcs])

    def test_screenshot_decorator(self):
        func = registry.screenshot_provider()
        expected = 'take_screenshot'
        self.assertEqual(expected, func.__name__)


class ScreenshotsTests(TestCase):
    def setUp(self):
        self.__old_screenshot_provider = registry.screenshot_provider()
        self.__is_screenshot_writer = registry.is_screenshot_writer
        os.environ["gauge_screenshots_dir"] = tempfile.mkdtemp()

    def test_pending_screenshots(self):
        ScreenshotsStore.capture()
        pending_screenshots = ScreenshotsStore.pending_screenshots()
        self.assertEqual(1, len(pending_screenshots))
        self.assertTrue(os.path.exists(os.path.join(
            os.getenv("gauge_screenshots_dir"), pending_screenshots[0])))

    def test_clear(self):
        ScreenshotsStore.capture()
        ScreenshotsStore.clear()
        pending_screenshots = ScreenshotsStore.pending_screenshots()
        self.assertEqual([], pending_screenshots)

    def test_pending_screenshots_gives_only_those_screenshots_which_are_not_collected(self):
        ScreenshotsStore.capture()
        pending_screenshots = ScreenshotsStore.pending_screenshots()
        self.assertEqual(1, len(pending_screenshots))
        screenshot_file = pending_screenshots[0]
        pending_screenshots = ScreenshotsStore.pending_screenshots()
        self.assertEqual(0, len(pending_screenshots))
        ScreenshotsStore.capture()
        pending_screenshots = ScreenshotsStore.pending_screenshots()
        self.assertEqual(1, len(pending_screenshots))
        self.assertNotEqual(screenshot_file, pending_screenshots[0])

    def test_capture_shoould_vefify_screenshot_file_for_file_based_custom_screenshot(self):
        first_screenshot = os.path.join(
            os.getenv("gauge_screenshots_dir"), "screenshot{0}.png".format(uuid1()))
        second_screenshot = os.path.join(
            os.getenv("gauge_screenshots_dir"), "screenshot{0}.png".format(uuid1()))

        def returns_abs_path():
            return first_screenshot

        def returns_base_ath():
            return os.path.basename(second_screenshot)
        registry.set_screenshot_provider(returns_abs_path, True)
        ScreenshotsStore.capture()
        self.assertEqual([os.path.basename(first_screenshot)],
                         ScreenshotsStore.pending_screenshots())

        registry.set_screenshot_provider(returns_base_ath, True)
        ScreenshotsStore.capture()
        self.assertEqual([os.path.basename(second_screenshot)],
                         ScreenshotsStore.pending_screenshots())

    def tearDown(self):
        registry.set_screenshot_provider(
            self.__old_screenshot_provider, self.__is_screenshot_writer)


if __name__ == '__main__':
    main()



import importlib
import inspect
import json
import os
import re
import shutil
import sys
import traceback
from contextlib import contextmanager
from os import path

from getgauge import logger
from getgauge.registry import registry
from getgauge.util import get_project_root, get_step_impl_dirs

project_root = get_project_root()
impl_dirs = get_step_impl_dirs()
env_dir = os.path.join(project_root, 'env', 'default')
requirements_file = os.path.join(project_root, 'requirements.txt')
sys.path.append(project_root)
temporary_sys_path = []
PLUGIN_JSON = 'python.json'
VERSION = 'version'
PYTHON_PROPERTIES = 'python.properties'
SKEL = 'skel'


def load_impls(step_impl_dirs=impl_dirs):
    os.chdir(project_root)
    for impl_dir in step_impl_dirs:
        if not os.path.isdir(impl_dir):
            logger.error('Cannot import step implementations. Error: {} does not exist.'.format(step_impl_dirs))
            logger.error('Make sure `STEP_IMPL_DIR` env var is set to a valid directory path.')
            return
        base_dir = project_root if impl_dir.startswith(project_root) else os.path.dirname(impl_dir)
        # Handle multi-level relative imports
        for _ in range(impl_dir.count('..')):
            base_dir = os.path.dirname(base_dir).replace("/", os.path.sep).replace("\\", os.path.sep)
        # Add temporary sys path for relative imports that is not already added
        if '..' in impl_dir and base_dir not in temporary_sys_path:
            temporary_sys_path.append(base_dir)
        _import_impl(base_dir, impl_dir)


def copy_skel_files():
    try:
        logger.info('Initialising Gauge Python project')
        logger.info('create  {}'.format(env_dir))
        os.makedirs(env_dir)
        logger.info('create  {}'.format(impl_dirs[0]))
        shutil.copytree(os.path.join(SKEL,path.basename(impl_dirs[0]) ), impl_dirs[0])
        logger.info('create  {}'.format(os.path.join(env_dir, PYTHON_PROPERTIES)))
        shutil.copy(os.path.join(SKEL, PYTHON_PROPERTIES), env_dir)
        with open(requirements_file, 'w', encoding="utf-8") as f:
            f.write('getgauge==' + _get_version())
    except:
        logger.fatal('Exception occurred while copying skel files.\n{}.'.format(traceback.format_exc()))


def _import_impl(base_dir, step_impl_dir):
    for f in os.listdir(step_impl_dir):
        file_path = os.path.join(step_impl_dir, f)
        if f.endswith('.py'):
            _import_file(base_dir, file_path)
        elif path.isdir(file_path):
            _import_impl(base_dir, file_path)

@contextmanager
def use_temporary_sys_path():
    original_sys_path = sys.path[:]
    sys.path.extend(temporary_sys_path)
    try:
        yield
    finally:
        sys.path = original_sys_path

def _import_file(base_dir, file_path):
    rel_path = os.path.normpath(file_path.replace(base_dir + os.path.sep, ''))
    try:
        module_name = os.path.splitext(rel_path.replace(os.path.sep, '.'))[0]
        # Use temporary sys path for relative imports
        if '..' in file_path:
            with use_temporary_sys_path():
                m = importlib.import_module(module_name)
        else:
            m = importlib.import_module(module_name)
        # Get all classes in the imported module
        classes = inspect.getmembers(m, lambda member: inspect.isclass(member) and member.__module__ == module_name)
        if len(classes) > 0:
            for c in classes:
                file = inspect.getfile(c[1])
                # Create instance of step implementation class.
                if _has_methods_with_gauge_decoratores(c[1]):
                    update_step_registry_with_class(c[1](), file_path) # c[1]() will create a new instance of the class
    except:
        logger.fatal('Exception occurred while loading step implementations from file: {}.\n{}'.format(rel_path, traceback.format_exc()))

# Inject instance in each class method (hook/step)
def update_step_registry_with_class(instance, file_path):
    # Resolve the absolute path from relative path
    # Note: relative path syntax ".." can appear in between the file_path too like "<Project_Root>/../../Other_Project/src/step_impl/file.py"
    file_path = os.path.abspath(file_path) if ".." in str(file_path) else file_path
    method_list = registry.get_all_methods_in(file_path)
    for info in method_list:
        class_methods = [x[0] for x in inspect.getmembers(instance, inspect.ismethod)]
        if info.impl.__name__ in class_methods:
            info.instance = instance
    return method_list

def _get_version():
    with open(PLUGIN_JSON, "r", encoding="utf-8") as json_data:
        data = json.loads(json_data.read())
    return data[VERSION]

def _has_methods_with_gauge_decoratores(klass) -> bool:
    gauge_decorator_pattern = r"@(step|before_suite|after_suite|before_scenario|after_scenario|before_spec|after_spec|before_step|after_step|screenshot|custom_screen_grabber)"
    sourcelines = inspect.getsourcelines(klass)[0]
    for line in sourcelines:
        if re.match(gauge_decorator_pattern, line.strip()) is not None:
            return True
    return False



from unittest import main, TestCase
from getgauge.util import get_step_impl_dirs
import os


class UtilTests(TestCase):

    def test_get_step_impl_gives_default_if_no_env_Set(self):
        dirs = get_step_impl_dirs()
        expected = ['step_impl']
        self.assertEqual(dirs,expected)

    def test_get_step_impl_returns_array_of_impl_dirs(self):
        os.environ["STEP_IMPL_DIR"] = "step_impl, step_impl1"
        dirs = get_step_impl_dirs()
        expected = ['step_impl','step_impl1']
        self.assertEqual(dirs,expected)

    def test_get_step_impl_returns_normalized_impl_dirs_path(self):
        os.environ["STEP_IMPL_DIR"] =  "test\\step_impl" if os.path.sep == '/' else "test/step_impl"
        dirs = get_step_impl_dirs()
        expected = ["test" + os.path.sep +"step_impl"]
        self.assertEqual(dirs,expected)

if __name__ == '__main__':
    main()


import os
from os import path
from textwrap import dedent
from unittest import main

from getgauge import processor
from getgauge import static_loader as loader
from getgauge.exceptions import SkipScenarioException
from getgauge.messages.messages_pb2 import *
from getgauge.messages.spec_pb2 import Parameter, ProtoExecutionResult, Span
from getgauge.parser import Parser
from getgauge.messages.spec_pb2 import ProtoStepValue
from getgauge.util import get_step_impl_dirs
from getgauge.python import data_store
from getgauge.registry import registry
from pyfakefs.fake_filesystem_unittest import TestCase


class ProcessorTests(TestCase):
    def setUp(self):
        self.setUpPyfakefs()
        data_store.suite.clear()
        data_store.spec.clear()
        data_store.scenario.clear()
        registry.clear()

    def tearDown(self):
        registry.clear()

    def load_content_steps(self, content):
        content = dedent(content)
        pf = Parser.parse("foo.py", content)
        self.assertIsNotNone(pf)
        loader.load_steps(pf)

    def test_Processor_suite_data_store_init_request(self):
        data_store.suite['suite'] = 'value'

        self.assertNotEqual(0, len(data_store.suite))

        response = processor.process_suite_data_store_init_request()

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)
        self.assertEqual(0, response.executionResult.executionTime)
        self.assertDictEqual({}, data_store.suite)

    def test_Processor_spec_data_store_init_request(self):
        data_store.spec['spec'] = 'value'

        self.assertNotEqual(0, len(data_store.spec))

        response = processor.process_spec_data_store_init_request()

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)
        self.assertEqual(0, response.executionResult.executionTime)
        self.assertDictEqual({}, data_store.spec)

    def test_Processor_scenario_data_store_init_request(self):
        data_store.scenario['scenario'] = 'value'

        self.assertNotEqual(0, len(data_store.scenario))

        response = processor.process_scenario_data_store_init_request()

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)
        self.assertEqual(0, response.executionResult.executionTime)
        self.assertDictEqual({}, data_store.scenario)

    def test_Processor_step_names_request(self):
        registry.add_step('Step <a> with <b>', 'func', '')
        registry.add_step('Step 4', 'func1', '')

        response = processor.process_step_names_request()

        self.assertIsInstance(response, StepNamesResponse)
        self.assertEqual({'Step <a> with <b>', 'Step 4'}, set(response.steps))

    def test_Processor_step_name_request(self):
        registry.add_step('Step <a> with <b>', 'func', '', {
                          'start': 1, 'startChar': 0, 'end': 3, 'endChar': 10})
        registry.add_step('Step 4', 'func1', '', {
                          'start': 5, 'startChar': 0, 'end': 6, 'endChar': 10})
        request = StepNameRequest()
        request.stepValue = 'Step {} with {}'

        response = processor.process_step_name_request(request)
        self.assertIsInstance(response, StepNameResponse)
        self.assertEqual(['Step <a> with <b>'], response.stepName)
        self.assertTrue(response.isStepPresent)
        request = StepNameRequest()
        self.assertFalse(response.hasAlias)

        request.stepValue = 'Step 4'
        response = processor.process_step_name_request(request)

        self.assertIsInstance(response, StepNameResponse)
        self.assertEqual(['Step 4'], response.stepName)
        self.assertTrue(response.isStepPresent)
        self.assertFalse(response.hasAlias)

    def test_Processor_step_name_request_with_aliases(self):
        registry.add_step(['Step 1', 'Step 2', 'Step 3'], 'func1', '',
                          {'start': 5, 'startChar': 0, 'end': 6, 'endChar': 10})
        request = StepNameRequest()
        request.stepValue = 'Step 1'

        response = processor.process_step_name_request(request)

        self.assertIsInstance(response, StepNameResponse)
        self.assertIn('Step 1', response.stepName)
        self.assertIn('Step 2', response.stepName)
        self.assertIn('Step 3', response.stepName)
        self.assertTrue(response.isStepPresent)
        self.assertTrue(response.hasAlias)

    def test_Processor_step_name_request_with_unimplemented_step(self):
        request = StepNameRequest()
        request.stepValue = 'Step {} with {}'

        response = processor.process_step_name_request(request)

        self.assertIsInstance(response, StepNameResponse)
        self.assertEqual([], response.stepName)
        self.assertFalse(response.isStepPresent)
        self.assertFalse(response.hasAlias)

    def test_Processor_valid_step_validate_request(self):
        registry.add_step('Step <a> with <b>', 'func', '')
        registry.add_step('Step 4', 'func1', '')

        request = StepValidateRequest()
        request.stepText = 'Step {} with {}'
        response = processor.process_validate_step_request(request)

        self.assertIsInstance(response, StepValidateResponse)
        self.assertTrue(response.isValid)

    def test_Processor_invalid_step_validate_request_when_no_impl_found(self):
        registry.add_step('Step <a> with <b>', 'func', '')
        registry.add_step('Step 4', 'func1', '')

        request = StepValidateRequest()
        request.stepText = 'Step2'
        request.stepValue.stepValue = 'Step2'

        response = processor.process_validate_step_request(request)

        self.assertIsInstance(response, StepValidateResponse)
        self.assertFalse(response.isValid)
        self.assertEqual(StepValidateResponse.STEP_IMPLEMENTATION_NOT_FOUND,
                         response.errorType)
        self.assertIn('@step("")\ndef step2():\n    assert False, "Add implementation code"', response.suggestion)

    def test_Processor_invalid_step_validate_request_when_duplicate_impl_found(self):
        registry.add_step('Step <a> with <b>', impl, '', {'start': 0})
        registry.add_step('Step <a> with <b>', impl, '', {'start': 2})

        response = Message()

        request = StepValidateRequest()
        request.stepText = 'Step {} with {}'

        response = processor.process_validate_step_request(request)

        self.assertIsInstance(response, StepValidateResponse)
        self.assertFalse(response.isValid)
        self.assertEqual(StepValidateResponse.DUPLICATE_STEP_IMPLEMENTATION,
                         response.errorType)

    def test_Processor_execute_step_request(self):
        registry.add_step('Step 4', impl1, '')

        request = ExecuteStepRequest()
        request.parsedStepText = 'Step 4'

        response = processor.process_execute_step_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)
        self.assertEqual(
            '', response.executionResult.errorMessage)
        self.assertEqual(
            '', response.executionResult.stackTrace)
        self.assertFalse(response.executionResult.skipScenario)
        self.assertEqual([], response.executionResult.message)

    def test_Processor_execute_step_request_with_param(self):
        registry.add_step('Step <a> with <b>', impl, '')
        registry.add_step('Step 4', 'func1', '')

        request = ExecuteStepRequest()
        request.parsedStepText = 'Step {} with {}'
        parameter = Parameter()
        parameter.value = 'param 1'
        parameter1 = Parameter()
        parameter1.value = 'param 2'
        request.parameters.extend([parameter, parameter1])

        response = processor.process_execute_step_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)
        self.assertEqual(
            '', response.executionResult.errorMessage)
        self.assertEqual(
            '', response.executionResult.stackTrace)
        self.assertFalse(response.executionResult.skipScenario)
        self.assertEqual([], response.executionResult.message)

    def test_Processor_failed_execute_step_request(self):
        request = ExecuteStepRequest()

        response = processor.process_execute_step_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertNotEqual(
            '', response.executionResult.errorMessage)
        self.assertNotEqual(
            '', response.executionResult.stackTrace)
        self.assertFalse(response.executionResult.recoverableError)
        self.assertFalse(response.executionResult.skipScenario)
        self.assertEqual([], response.executionResult.message)

    def test_Processor_failed_execute_step_request_with_continue_on_failure(self):
        registry.add_step('Step 4', failing_impl, '')
        registry.continue_on_failure(failing_impl, [IndexError])

        request = ExecuteStepRequest()
        request.parsedStepText = 'Step 4'

        response = processor.process_execute_step_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertNotEqual('', response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)
        self.assertTrue(response.executionResult.recoverableError)
        self.assertFalse(response.executionResult.skipScenario)
        self.assertEqual([], response.executionResult.message)

    def test_Processor_failed_execute_step_request_with_programmatic_skip(self):
        registry.add_step('Skipped Step', skipped_impl, '')

        request = ExecuteStepRequest()
        request.parsedStepText = 'Skipped Step'

        response = processor.process_execute_step_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)
        self.assertEqual(
            '', response.executionResult.errorMessage)
        self.assertEqual(
            '', response.executionResult.stackTrace)
        self.assertTrue(response.executionResult.skipScenario)
        self.assertEqual(['Step programmatically skipped'], response.executionResult.message)

    def test_Processor_starting_execution_request(self):
        registry.add_before_suite(impl1)
        registry.add_before_suite(impl2)
        request = ExecutionStartingRequest()
        response = processor.process_execution_starting_request(request, False)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_ending_execution_request(self):
        registry.add_after_suite(impl1)
        registry.add_after_suite(impl2)
        request = ExecutionEndingRequest()
        response = processor.process_execution_ending_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_spec_starting_execution_request(self):
        registry.add_before_spec(impl1)
        registry.add_before_spec(impl2)
        request = SpecExecutionEndingRequest()
        response = processor.process_spec_execution_starting_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_spec_ending_execution_request(self):
        registry.add_after_spec(impl1)
        registry.add_after_spec(impl2)
        request = SpecExecutionEndingRequest()
        response = processor.process_spec_execution_ending_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_scenario_starting_execution_request(self):
        registry.add_before_scenario(impl1)
        registry.add_before_scenario(impl2)
        request = ScenarioExecutionStartingRequest()
        response = processor.process_scenario_execution_starting_request(
            request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_scenario_ending_execution_request(self):
        registry.add_after_scenario(impl1)
        registry.add_after_scenario(impl2)
        request = ScenarioExecutionEndingRequest()
        response = processor.process_scenario_execution_ending_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_step_starting_execution_request(self):
        registry.add_before_step(impl1)
        registry.add_before_step(impl2)
        request = StepExecutionStartingRequest()
        response = processor.process_step_execution_starting_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_step_ending_execution_request(self):
        registry.add_after_step(impl1)
        registry.add_after_step(impl2)
        request = StepExecutionEndingRequest()
        response = processor.process_step_execution_ending_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertFalse(response.executionResult.failed)

    def test_Processor_failing_starting_execution_request(self):
        registry.add_before_suite(failing_impl)
        request = ExecutionStartingRequest()
        response = processor.process_execution_starting_request(
            request, False)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_ending_execution_request(self):
        registry.add_after_suite(failing_impl)
        request = ExecutionEndingRequest()
        response = processor.process_execution_ending_request(request)
        print(response)
        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_spec_starting_execution_request(self):
        registry.add_before_spec(failing_impl)
        request = SpecExecutionStartingRequest()
        response = processor.process_spec_execution_starting_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_spec_ending_execution_request(self):
        registry.add_after_spec(failing_impl)
        request = SpecExecutionEndingRequest()
        response = processor.process_spec_execution_ending_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_scenario_starting_execution_request(self):
        registry.add_before_scenario(failing_impl)
        request = ScenarioExecutionStartingRequest()
        response = processor.process_scenario_execution_starting_request(
            request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_scenario_ending_execution_request(self):
        registry.add_after_scenario(failing_impl)
        request = ScenarioExecutionEndingRequest()
        response = processor.process_scenario_execution_ending_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_step_starting_execution_request(self):
        registry.add_before_step(failing_impl)
        request = StepExecutionStartingRequest()
        response = processor.process_step_execution_starting_request(request)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_failing_step_ending_execution_request(self):
        registry.add_after_step(failing_impl)
        request = StepExecutionEndingRequest()
        response = processor.process_step_execution_ending_request(request,)

        self.assertIsInstance(response, ExecutionStatusResponse)
        self.assertTrue(response.executionResult.failed)
        self.assertEqual(ProtoExecutionResult.ASSERTION,
                         response.executionResult.errorType)
        self.assertEqual('list index out of range',
                         response.executionResult.errorMessage)
        self.assertNotEqual('', response.executionResult.stackTrace)

    def test_Processor_refactor_request_when_multiple_impl_found(self):
        registry.add_step('Step <a> with <b>', 'func', '')
        registry.add_step('Step <a> with <b>', 'func', '')
        request = RefactorRequest()
        request.oldStepValue.stepValue = 'Step {} with {}'
        request.oldStepValue.parameterizedStepValue = 'Step <a> with <b>'

        response = processor.process_refactor_request(request)

        self.assertIsInstance(response, RefactorResponse)
        self.assertFalse(response.success)
        self.assertEqual(
            'Reason: Multiple Implementation found for `Step <a> with <b>`', response.error)

    def test_Processor_step_position_request(self):

        registry.add_step('Step <a> with <b>', 'func', 'foo.py',
                          {'start': 0, 'startChar': 0, 'end': 3, 'endChar': 10})
        registry.add_step('Step 1', 'func', 'foo.py', {
                          'start': 4, 'startChar': 0, 'end': 7, 'endChar': 10})

        request = StepPositionsRequest()
        request.filePath = 'foo.py'

        response = processor.process_step_positions_request(request)

        self.assertIsInstance(response, StepPositionsResponse)
        self.assertEqual('', response.error)

        steps = [(p.stepValue, p.span.start)
                 for p in response.stepPositions]

        self.assertIn(('Step {} with {}', 0), steps)
        self.assertIn(('Step 1', 4), steps)

    def test_Processor_put_stub_impl_request(self):
        request = StubImplementationCodeRequest()

        codes = ["code1", "code2"]
        request.implementationFilePath = ""
        request.codes.extend(codes)

        response = processor.process_stub_impl_request(request)

        expected_output_codes = dedent('''\
        from getgauge.python import step

        code1
        code2''')
        expected_span = Span(
            **{'start': 0, 'startChar': 0, 'end': 0, 'endChar': 0})
        expected_text_diff = TextDiff(
            **{'span': expected_span, 'content': expected_output_codes})

        self.assertEqual(len(response.textDiffs), 1)
        self.assertEqual(response.textDiffs[0], expected_text_diff)
        self.assertEqual(path.basename(
            response.filePath), "step_implementation.py")

    def test_Processor_glob_pattern(self):
        request = ImplementationFileGlobPatternRequest()
        response = processor.process_glob_pattern_request(request)
        self.assertEqual(response.globPatterns, ["step_impl/**/*.py"])

    def test_Processor_cache_file_with_opened_status(self):
        request = CacheFileRequest()
        self.load_content_steps('''\
        from getgauge.python import step

        @step('foo1')
        def foo():
            pass
        ''')

        request.filePath = 'foo.py'
        request.content = dedent('''\
        from getgauge.python import step

        @step('foo <bar>')
        def foo():
            pass
        ''')
        request.status = CacheFileRequest.OPENED

        processor.process_cache_file_request(request)

        self.assertEqual(registry.is_implemented('foo1'), False)
        self.assertEqual(registry.is_implemented('foo {}'), True)

    def test_Processor_cache_file_with_changed_status(self):
        request = CacheFileRequest()
        self.load_content_steps('''\
        from getgauge.python import step

        @step('foo1')
        def foo():
            pass
        ''')

        request.filePath = 'foo.py'
        request.content = dedent('''\
        from getgauge.python import step

        @step('foo <bar>')
        def foo():
            pass
        ''')
        request.status = CacheFileRequest.CHANGED

        processor.process_cache_file_request(request)

        self.assertFalse(registry.is_implemented('foo1'))
        self.assertTrue(registry.is_implemented('foo {}'))

    def test_Processor_cache_file_with_create_status(self):
        request = CacheFileRequest()

        request.filePath = 'foo.py'
        request.status = CacheFileRequest.CREATED
        self.fs.create_file('foo.py', contents=dedent('''\
        from getgauge.python import step

        @step('foo <bar>')
        def foo():
            pass
        '''))
        processor.process_cache_file_request(request)

        self.assertTrue(registry.is_implemented('foo {}'))

    def test_Processor_cache_file_with_create_status_when_file_is_cached(self):
        request = CacheFileRequest()
        self.load_content_steps('''\
        from getgauge.python import step

        @step('foo <bar>')
        def foo():
            pass
        ''')

        self.assertTrue(registry.is_implemented('foo {}'))

        request.filePath = 'foo.py'
        request.status = CacheFileRequest.CREATED
        self.fs.create_file('foo.py')
        processor.process_cache_file_request(request)

        self.assertTrue(registry.is_implemented('foo {}'))

    def test_Processor_cache_file_with_closed_status(self):
        request = CacheFileRequest()

        self.load_content_steps('''\
        from getgauge.python import step

        @step('foo1')
        def foo():
            pass
        ''')

        request.filePath = 'foo.py'
        request.status = CacheFileRequest.CLOSED
        self.fs.create_file('foo.py', contents=dedent('''\
        from getgauge.python import step

        @step('foo <bar>')
        def foo():
            pass
        '''))
        processor.process_cache_file_request(request)

        self.assertFalse(registry.is_implemented('foo1'))
        self.assertTrue(registry.is_implemented('foo {}'))

    def test_Processor_cache_file_with_delete_status(self):
        request = CacheFileRequest()
        self.load_content_steps('''\
        from getgauge.python import step

        @step('foo1')
        def foo():
            pass
        ''')

        request.filePath = 'foo.py'
        request.status = CacheFileRequest.DELETED

        processor.process_cache_file_request(request)

        self.assertFalse(registry.is_implemented('foo1'))

    def test_Processor_process_impl_files_request(self):
        self.fs.create_file(os.path.join(get_step_impl_dirs()[0], 'foo.py'))
        res = processor.process_impl_files_request()
        self.assertEqual(os.path.basename(
            res.implementationFilePaths[0]), 'foo.py')

    def test_Processor_process_step_names_request(self):
        self.load_content_steps('''\
        @step('foo')
        def foo():
            pass
        ''')

        res = processor.process_step_names_request()
        self.assertEqual(res.steps, ['foo'])

    def test_Processor_process_step_name_request(self):
        self.load_content_steps('''\
        @step('foo')
        def foo():
            pass
        ''')

        req = StepNameRequest(stepValue='foo')
        res = processor.process_step_name_request(req)

        self.assertTrue(res.isStepPresent)
        self.assertEqual(res.fileName, 'foo.py')

    def test_Processor_process_validate_step_request(self):
        self.load_content_steps('''\
        @step('foo')
        def foo():
            pass
        ''')
        step_value = ProtoStepValue(
            stepValue='foo', parameterizedStepValue='foo')

        req = StepValidateRequest(
            stepText='foo', stepValue=step_value, numberOfParameters=0)
        res = processor.process_validate_step_request(req)
        self.assertTrue(res.isValid)

    def test_Prcessor_process_step_positions_request(self):
        self.load_content_steps('''\
        @step('foo')
        def foo():
            pass
        ''')

        req = StepPositionsRequest(filePath='foo.py')
        res = processor.process_step_positions_request(req)
        self.assertEqual(res.stepPositions[0].stepValue, 'foo')

    def test_Processor_process_implement_stub_request(self):
        self.load_content_steps("@step('foo')\ndef foo():\n\tpass\n")

        req = StubImplementationCodeRequest(
            implementationFilePath='New File', codes=['add hello'])
        res = processor.process_stub_impl_request(req)
        self.assertEqual(os.path.basename(res.filePath),
                         'step_implementation.py')
        self.assertEqual(
            res.textDiffs[0].content, 'from getgauge.python import step\n\nadd hello')

    def test_Processor_process_refactor_request(self):
        content = dedent('''\
        from getgauge.python import step

        @step('Vowels in English language are <aeiou>.')
        def foo(vowels):
            print(vowels)
        ''')
        self.fs.create_file(os.path.join(
            get_step_impl_dirs()[0], 'foo.py'), contents=content)
        loader.load_files(get_step_impl_dirs())

        request = RefactorRequest()
        request.saveChanges = False
        request.oldStepValue.stepValue = 'Vowels in English language are {}.'
        request.oldStepValue.parameters.append('vowels')
        request.newStepValue.parameterizedStepValue = 'Vowels in English language is <vowels> <bsdfdsf>.'
        request.newStepValue.stepValue = 'Vowels in English language is {} {}.'
        request.newStepValue.parameters.extend(['vowels', 'bsdfdsf'])
        position = ParameterPosition()
        position.oldPosition = 0
        position.newPosition = 0
        param_position = ParameterPosition()
        param_position.oldPosition = -1
        param_position.newPosition = 1
        request.paramPositions.extend([position, param_position])

        res = processor.process_refactor_request(request)

        self.assertTrue(res.success)
        diff_contents = [diff.content for diff in res.fileChanges[0].diffs]
        self.assertIn("vowels, arg1", diff_contents)
        self.assertIn(
            "'Vowels in English language is <vowels> <bsdfdsf>.'", diff_contents)

    def test_Processor_process_cache_file_request(self):
        self.load_content_steps('''\
        from getgauge.python import step

        @step('Vowels in English language are <aeiou>.')
        def foo(vowels):
            print(vowels)
        ''')

        self.assertTrue(registry.is_implemented(
            'Vowels in English language are {}.'))

        content = dedent('''\
        from getgauge.python import step

        @step('get lost!')
        def foo():
            pass
        ''')
        req = CacheFileRequest(
            content=content, filePath='foo.py', status=CacheFileRequest.CHANGED)
        processor.process_cache_file_request(req)

        self.assertTrue(registry.is_implemented('get lost!'))


def impl(a, b):
    pass


def impl1():
    pass


def impl2(context):
    pass


def failing_impl():
    print([][1])

def skipped_impl():
    raise SkipScenarioException("Step programmatically skipped")


if __name__ == '__main__':
    main()



from os import path

from getgauge.executor import (create_execution_status_response,
                               execute_method, run_hook)
from getgauge.impl_loader import load_impls
from getgauge.messages.messages_pb2 import *
from getgauge.messages.spec_pb2 import Parameter, Span
from getgauge.python import Table, create_execution_context_from, data_store
from getgauge.refactor import refactor_step
from getgauge.registry import MessagesStore, ScreenshotsStore, registry
from getgauge.static_loader import reload_steps
from getgauge.util import (get_file_name, get_impl_files, get_step_impl_dirs,
                           read_file_contents)
from getgauge.validator import validate_step


def process_validate_step_request(request):
    return validate_step(request)


def process_step_name_request(request):
    response = StepNameResponse()
    info = registry.get_info_for(request.stepValue)
    response.isStepPresent = False
    if info.step_text is not None:
        response.isStepPresent = True
        if info.has_alias:
            for alias in info.aliases:
                response.stepName.append(alias)
        else:
            response.stepName.append(info.step_text)
        response.fileName = info.file_name
        response.span.start = info.span['start']
        response.span.startChar = info.span['startChar']
        response.span.end = info.span['end']
        response.span.endChar = info.span['endChar']
    response.hasAlias = info.has_alias
    return response


def process_refactor_request(request):
    response = RefactorResponse()
    try:
        refactor_step(request, response)
    except Exception as e:
        response.success = False
        response.error = f"Reason: {e}"
    return response


def process_step_names_request():
    response = StepNamesResponse()
    response.steps.extend(registry.steps())
    return response


def process_execute_step_request(request):
    params = []
    for p in request.parameters:
        params.append(Table(p.table) if p.parameterType in [
            Parameter.Table, Parameter.Special_Table] else p.value)
    response = create_execution_status_response()
    info = registry.get_info_for(request.parsedStepText)
    execute_method(params, info, response, registry.is_continue_on_failure)
    return response


def _add_message_and_screenshots(response):
    response.executionResult.message.extend(MessagesStore.pending_messages())
    response.executionResult.screenshots.extend(
        ScreenshotsStore.pending_screenshots())


def process_execution_starting_request(request, clear=True):
    if clear:
        registry.clear()
        load_impls(get_step_impl_dirs())
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    response = run_hook(request, registry.before_suite(), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_execution_ending_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    response = run_hook(request, registry.after_suite(), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_spec_execution_starting_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    response = run_hook(request, registry.before_spec(
        execution_info.specification.tags), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_spec_execution_ending_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    response = run_hook(request, registry.after_spec(
        execution_info.specification.tags), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_scenario_execution_starting_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    tags = list(execution_info.scenario.tags) + \
        list(execution_info.specification.tags)
    response = run_hook(
        request, registry.before_scenario(tags), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_scenario_execution_ending_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    tags = list(execution_info.scenario.tags) + \
        list(execution_info.specification.tags)
    response = run_hook(request, registry.after_scenario(tags), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_step_execution_starting_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    tags = list(execution_info.scenario.tags) + \
        list(execution_info.specification.tags)
    response = run_hook(request, registry.before_step(tags), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_step_execution_ending_request(request):
    execution_info = create_execution_context_from(
        request.currentExecutionInfo)
    tags = list(execution_info.scenario.tags) + \
        list(execution_info.specification.tags)
    response = run_hook(request, registry.after_step(tags), execution_info)
    _add_message_and_screenshots(response)
    return response


def process_scenario_data_store_init_request():
    data_store.scenario.clear()
    return create_execution_status_response()


def process_spec_data_store_init_request():
    data_store.spec.clear()
    return create_execution_status_response()


def process_suite_data_store_init_request():
    data_store.suite.clear()
    return create_execution_status_response()


def _load_from_disk(file_path):
    if path.isfile(file_path):
        reload_steps(file_path)


def process_cache_file_request(request):
    file = request.filePath
    status = request.status
    if status in [CacheFileRequest.CHANGED, CacheFileRequest.OPENED]:
        reload_steps(file, request.content)
    elif status == CacheFileRequest.CREATED:
        if not registry.is_file_cached(file):
            _load_from_disk(file)
    elif status == CacheFileRequest.CLOSED:
        _load_from_disk(file)
    else:
        registry.remove_steps(file)
    return Empty()


def process_step_positions_request(request):
    file_path = request.filePath
    positions = registry.get_step_positions(file_path)
    response = StepPositionsResponse()
    response.stepPositions.extend([_create_pos(x) for x in positions])
    return response


def _create_pos(p):
    return StepPositionsResponse.StepPosition(**{'stepValue': p['stepValue'], 'span': Span(**p['span'])})


def process_impl_files_request():
    files = get_impl_files()
    res = ImplementationFileListResponse()
    res.implementationFilePaths.extend(files)
    return res


def process_stub_impl_request(request):
    response = FileDiff()
    file_name = request.implementationFilePath
    codes = request.codes
    content = read_file_contents(file_name)
    prefix = ""
    if content is not None:
        new_line_char = '\n' if len(content.strip().split(
            '\n')) == len(content.split('\n')) else ''
        last_line = len(content.split('\n'))
        prefix = "from getgauge.python import step\n" if len(
            content.strip()) == 0 else new_line_char
        span = Span(**{'start': last_line, 'startChar': 0,
                       'end': last_line, 'endChar': 0})
    else:
        file_name = get_file_name()
        prefix = "from getgauge.python import step\n"
        span = Span(**{'start': 0, 'startChar': 0, 'end': 0, 'endChar': 0})
    codes = [prefix] + codes[:]
    text_diffs = [TextDiff(**{'span': span, 'content': '\n'.join(codes)})]
    response.filePath = file_name
    response.textDiffs.extend(text_diffs)
    return response


def process_glob_pattern_request(request):
    res = ImplementationFileGlobPatternResponse()
    glob_patterns = [[f"{d}/**/*.py"] for d in get_step_impl_dirs()]
    res.globPatterns.extend(
        [item for sublist in glob_patterns for item in sublist])
    return res



import sys
import unittest
from textwrap import dedent
from getgauge.parser import Parser


def _span(start, startChar, end, endChar):
    return locals()


class ParserTests(unittest.TestCase):
    def setUp(self):
        self.preservesNewlines = False

    def parse(self, content, file_path='foo.py'):
        return Parser.parse(file_path, content)
    
    def assertSpanStart(self, actualSpan, expectedStartLine, expectedStartChar):
        if callable(actualSpan):
            actualSpan = actualSpan()
        self.assertEqual(actualSpan['start'], expectedStartLine)
        self.assertEqual(actualSpan['startChar'], expectedStartChar)

    def test_can_parse_content_directly(self):
        content = dedent('''\
        class test:
            @step("print hello")
            def print_hello():
                print("hello")

            @step('print <hello>.')
            def print_word(word):
                print(word)

        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        self.assertEqual(pf.get_code(), content)

    def test_does_not_parse_invalid_code(self):
        content = dedent('''\
        @step("print hello")
        def print_hello():
            print("hello)
        ''')
        pf = self.parse(content)
        self.assertIsNone(pf)

    def test_can_parse_chinese_content_directly(self):
        content = dedent('''\
        class test:
            @step("打印你好")
            def print_hello():
                print("hello")

            @step('打印 <hello>.')
            def print_word(word):
                print(word)

        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        self.assertEqual(pf.get_code(), content)

    def test_iter_steps_loads_steps_from_content(self):
        content = dedent('''\
        @step("print hello")
        def print_hello():
            print("hello")

        @step("print <word>")
        def print_word(word):
            print(word)
        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 2)
        self.assertEqual(steps[0][0], "print hello")
        self.assertEqual(steps[0][1], "print_hello")
        self.assertSpanStart(steps[0][2], 1, 0)
        self.assertEqual(steps[1][0], "print <word>")
        self.assertEqual(steps[1][1], "print_word")
        self.assertSpanStart(steps[1][2], 5, 0)

    def test_iter_steps_loads_multiple_steps_from_list(self):
        content = dedent('''\
        @step(["print hello", "display hello"])
        def print_hello():
            print("hello")

        @step([
            "print <word>",
            'display <word>',
        ])
        def print_word(word):
            print(word)
        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 2)
        self.assertEqual(steps[0][0], ["print hello", "display hello"])
        self.assertEqual(steps[0][1], "print_hello")
        self.assertSpanStart(steps[0][2], 1, 0)
        self.assertEqual(steps[1][0], ["print <word>", "display <word>"])
        self.assertEqual(steps[1][1], "print_word")
        self.assertSpanStart(steps[1][2], 5, 0)

    def test_iter_steps_only_checks_step_decorator(self):
        content = dedent('''\
        @step("print <word>")
        @Step("hello")
        def print_word(word):
            print(word)

        @_step("print <word>")
        def print_words(word):
            print(word)
        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 1)
        self.assertEqual(steps[0][0], "print <word>")
        self.assertEqual(steps[0][1], "print_word")
        self.assertSpanStart(steps[0][2], 1, 0)

    def test_iter_steps_loads_multiple_step_implementations(self):
        content = dedent('''\
        @step("print hello")
        def print_hello():
            print("hello")

        @step("print hello")
        def print_word(word):
            print(word)
        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 2)
        self.assertEqual(steps[0][0], "print hello")
        self.assertEqual(steps[0][1], "print_hello")
        self.assertSpanStart(steps[0][2], 1, 0)
        self.assertEqual(steps[1][0], "print hello")
        self.assertEqual(steps[1][1], "print_word")
        self.assertSpanStart(steps[1][2], 5, 0)

    def test_iter_steps_loads_triple_quoted_strings_1(self):
        content = dedent('''\
        @step("""print hello""")
        def print_hello():
            print("hello")
        ''')
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 1)
        self.assertEqual(steps[0][0], "print hello")
        self.assertEqual(steps[0][1], "print_hello")
        self.assertSpanStart(steps[0][2], 1, 0)

    def test_iter_steps_loads_triple_quoted_strings_2(self):
        content = dedent("""\
        @step('''print hello''')
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 1)
        self.assertEqual(steps[0][0], "print hello")
        self.assertEqual(steps[0][1], "print_hello")
        self.assertSpanStart(steps[0][2], 1, 0)

    def test_iter_steps_loads_r_strings(self):
        content = dedent("""\
        @step(r'''print hello''')
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 1)
        self.assertEqual(steps[0][0], "print hello")
        self.assertEqual(steps[0][1], "print_hello")
        self.assertSpanStart(steps[0][2], 1, 0)

    @unittest.skipIf(sys.hexversion < 0x3060000, "f-strings are introduced in python3.6")
    def test_iter_steps_does_not_load_f_strings(self):
        content = dedent("""\
        @step(f'print hello')
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 0)

    def test_iter_steps_does_not_load_indirect_args(self):
        content = dedent("""\
        s = 'print hello'

        @step(s)
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 0)

    def test_iter_steps_does_not_load_non_string_arg(self):
        content = dedent("""\
        @step(100)
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)
        steps = list(pf.iter_steps())
        self.assertEqual(len(steps), 0)

    def test_refactor_step_only_step_text(self):
        content = dedent("""\
        @step("print hello")
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step('print hello', 'display hello', [])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 19), '"display hello"'),
        ])
        self.assertEqual(pf.get_code(), content.replace(
            'print hello', 'display hello'))

    def test_refactor_step_only_step_text_in_list(self):
        content = dedent("""\
        @step([
            "print <word>",
            'display <word>',
        ])
        def print_word(word):
            print(word)
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step('display <word>', 'show <word>', [0])
        self.assertEqual(diffs, [
            (_span(3, 4, 3, 20), "'show <word>'"),
        ])
        self.assertEqual(pf.get_code(), content.replace(
            'display <word>', 'show <word>'))

    def test_refactor_step_add_arg_in_empty(self):
        content = dedent("""\
        @step('print hello')
        def print_hello():
            print("hello")
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step('print hello', 'print <word>', [-1])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 19), "'print <word>'"),
            (_span(2, 16, 2, 16), "arg0"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step('print <word>')
        def print_hello(arg0):
            print("hello")
        """))

    def test_refactor_step_add_arg_at_start(self):
        content = dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a, b, z):
            assert z == a * b
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <x>, <a>, <b> equals <z>',
            [-1, 0, 1, 2])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <x>, <a>, <b> equals <z>"'),
            (_span(2, 8, 2, 15), "arg0, a, b, z"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <x>, <a>, <b> equals <z>")
        def mul(arg0, a, b, z):
            assert z == a * b
        """))

    def test_refactor_step_add_arg_at_end(self):
        content = dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a, b, z):
            assert z == a * b
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <a>, <b>, <x> equals <z>',
            [0, 1, 2, -1])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <a>, <b>, <x> equals <z>"'),
            (_span(2, 8, 2, 15), "a, b, z, arg3"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <b>, <x> equals <z>")
        def mul(a, b, z, arg3):
            assert z == a * b
        """))

    def test_refactor_step_add_arg_in_middle(self):
        content = dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a, b, z):
            assert z == a * b
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <a>, <b>, <x> equals <z>',
            [0, 1, -1, 2])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <a>, <b>, <x> equals <z>"'),
            (_span(2, 8, 2, 15), "a, b, arg2, z"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <b>, <x> equals <z>")
        def mul(a, b, arg2, z):
            assert z == a * b
        """))

    def test_refactor_step_add_arg_at_start_newline(self):
        content = dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a,
                b,
                z):
            assert z == a * b
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <x>, <a>, <b> equals <z>',
            [-1, 0, 1, 2])

        expectedArgs = "arg0,\n        a,\n        b,\n        z"
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <x>, <a>, <b> equals <z>"'),
            (_span(2, 8, 4, 9), expectedArgs),
        ])

        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <x>, <a>, <b> equals <z>")
        def mul({}):
            assert z == a * b
        """).format(expectedArgs))

    def test_refactor_step_add_arg_at_end_newline(self):
        content = dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a,
                b,
                z):
            assert z == a * b
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <x>, <a>, <b> equals <z>',
            [0, 1, 2, -1])

        expectedArgs = "a,\n        b,\n        z,\n        arg3"
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <x>, <a>, <b> equals <z>"'),
            (_span(2, 8, 4, 9), expectedArgs),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <x>, <a>, <b> equals <z>")
        def mul({}):
            assert z == a * b
        """).format(expectedArgs))

    def test_refactor_step_add_arg_in_middle_newline(self):
        content = dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a,
                b,
                z):
            assert z == a * b
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <a>, <x>, <b> equals <z>',
            [0, -1, 1, 2])

        expectedArgs = "a,\n        arg1,\n        b,\n        z"
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <a>, <x>, <b> equals <z>"'),
            (_span(2, 8, 4, 9), expectedArgs),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <x>, <b> equals <z>")
        def mul({}):
            assert z == a * b
        """).format(expectedArgs))

    def test_refactor_step_remove_only_arg(self):
        content = dedent("""\
        @step("print <word>")
        def print_word(word):
            print(word)
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step('print <word>', 'print hello', [])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 20), '"print hello"'),
            (_span(2, 15, 2, 19), ""),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("print hello")
        def print_word():
            print(word)
        """))

    def test_refactor_step_remove_first_arg(self):
        content = dedent("""\
        @step("multiply <a>, <b>, <c> equals <z>")
        def mul(a, b, c, z):
            assert z == a * b * c
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b>, <c> equals <z>',
            'multiply <b>, <c> equals <z>',
            [1, 2, 3])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 41), '"multiply <b>, <c> equals <z>"'),
            (_span(2, 8, 2, 18), "b, c, z"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <b>, <c> equals <z>")
        def mul(b, c, z):
            assert z == a * b * c
        """))

    def test_refactor_step_remove_last_arg(self):
        content = dedent("""\
        @step("multiply <a>, <b>, <c> equals <z>")
        def mul(a, b, c, z):
            assert z == a * b * c
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b>, <c> equals <z>',
            'multiply <a>, <b> equals <c>',
            [0, 1, 2])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 41), '"multiply <a>, <b> equals <c>"'),
            (_span(2, 8, 2, 18), "a, b, c"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <b> equals <c>")
        def mul(a, b, c):
            assert z == a * b * c
        """))

    def test_refactor_step_remove_arg_in_middle(self):
        content = dedent("""\
        @step("multiply <a>, <b>, <c> equals <z>")
        def mul(a, b, c, z):
            assert z == a * b * c
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b>, <c> equals <z>',
            'multiply <a>, <b> equals <z>',
            [0, 1, 3])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 41), '"multiply <a>, <b> equals <z>"'),
            (_span(2, 8, 2, 18), "a, b, z"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul(a, b, z):
            assert z == a * b * c
        """))
        # Remove one more arg
        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <a> equals <z>',
            [0, 2])
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <a> equals <z>"'),
            (_span(2, 8, 2, 15), "a, z"),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a> equals <z>")
        def mul(a, z):
            assert z == a * b * c
        """))

    def test_refactor_step_remove_first_arg_newline(self):
        content = dedent("""\
        @step("multiply <a>, <b>, <c> equals <z>")
        def mul(a,
                b,
                c,
                z):
            assert z == a * b * c
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b>, <c> equals <z>',
            'multiply <b>, <c> equals <z>',
            [1, 2, 3])

        expectedArgs = "b,\n        c,\n        z"
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 41), '"multiply <b>, <c> equals <z>"'),
            (_span(2, 8, 5, 9), expectedArgs),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <b>, <c> equals <z>")
        def mul({}):
            assert z == a * b * c
        """).format(expectedArgs))

    def test_refactor_step_remove_last_arg_newline(self):
        content = dedent("""\
        @step("multiply <a>, <b>, <c> equals <z>")
        def mul(a,
                b,
                c,
                z):
            assert z == a * b * c
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b>, <c> equals <z>',
            'multiply <a>, <b> equals <c>',
            [0, 1, 2])

        expectedArgs = "a,\n        b,\n        c"
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 41), '"multiply <a>, <b> equals <c>"'),
            (_span(2, 8, 5, 9), expectedArgs),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <b> equals <c>")
        def mul({}):
            assert z == a * b * c
        """).format(expectedArgs))

    def test_refactor_step_remove_arg_in_middle_newline(self):
        content = dedent("""\
        @step("multiply <a>, <b>, <c> equals <z>")
        def mul(a,
                b,
                c,
                z):
            assert z == a * b * c
        """)
        pf = self.parse(content)
        self.assertIsNotNone(pf)

        diffs = pf.refactor_step(
            'multiply <a>, <b>, <c> equals <z>',
            'multiply <a>, <b> equals <z>',
            [0, 1, 3])

        expectedArgs = "a,\n        b,\n        z"
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 41), '"multiply <a>, <b> equals <z>"'),
            (_span(2, 8, 5, 9), expectedArgs),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a>, <b> equals <z>")
        def mul({}):
            assert z == a * b * c
        """).format(expectedArgs))

        # Remove one more arg
        diffs = pf.refactor_step(
            'multiply <a>, <b> equals <z>',
            'multiply <a> equals <z>',
            [0, 2])
        expectedArgs = "a,\n        z"
        expectedArgSpan = _span(2, 8, 2, 31)
        if not self.preservesNewlines:
            expectedArgs = expectedArgs.replace('\n       ', '')
            expectedArgSpan = _span(2, 8, 2, 15)
        self.assertEqual(diffs, [
            (_span(1, 6, 1, 36), '"multiply <a> equals <z>"'),
            (expectedArgSpan, expectedArgs),
        ])
        self.assertEqual(pf.get_code(), dedent("""\
        @step("multiply <a> equals <z>")
        def mul({}):
            assert z == a * b * c
        """).format(expectedArgs))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: spec.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nspec.proto\x12\x0egauge.messages\"\xf9\x03\n\tProtoSpec\x12\x13\n\x0bspecHeading\x18\x01 \x01(\t\x12(\n\x05items\x18\x02 \x03(\x0b\x32\x19.gauge.messages.ProtoItem\x12\x15\n\risTableDriven\x18\x03 \x01(\x08\x12\x39\n\x0fpreHookFailures\x18\x04 \x03(\x0b\x32 .gauge.messages.ProtoHookFailure\x12:\n\x10postHookFailures\x18\x05 \x03(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x10\n\x08\x66ileName\x18\x06 \x01(\t\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x17\n\x0fpreHookMessages\x18\x08 \x03(\t\x12\x18\n\x10postHookMessages\x18\t \x03(\t\x12\x1a\n\x0epreHookMessage\x18\n \x03(\tB\x02\x18\x01\x12\x1b\n\x0fpostHookMessage\x18\x0b \x03(\tB\x02\x18\x01\x12\x1e\n\x12preHookScreenshots\x18\x0c \x03(\x0c\x42\x02\x18\x01\x12\x1f\n\x13postHookScreenshots\x18\r \x03(\x0c\x42\x02\x18\x01\x12\x11\n\titemCount\x18\x0e \x01(\x03\x12\x1e\n\x16preHookScreenshotFiles\x18\x0f \x03(\t\x12\x1f\n\x17postHookScreenshotFiles\x18\x10 \x03(\t\"\x92\x04\n\tProtoItem\x12\x34\n\x08itemType\x18\x01 \x01(\x0e\x32\".gauge.messages.ProtoItem.ItemType\x12\'\n\x04step\x18\x02 \x01(\x0b\x32\x19.gauge.messages.ProtoStep\x12-\n\x07\x63oncept\x18\x03 \x01(\x0b\x32\x1c.gauge.messages.ProtoConcept\x12/\n\x08scenario\x18\x04 \x01(\x0b\x32\x1d.gauge.messages.ProtoScenario\x12\x45\n\x13tableDrivenScenario\x18\x05 \x01(\x0b\x32(.gauge.messages.ProtoTableDrivenScenario\x12-\n\x07\x63omment\x18\x06 \x01(\x0b\x32\x1c.gauge.messages.ProtoComment\x12)\n\x05table\x18\x07 \x01(\x0b\x32\x1a.gauge.messages.ProtoTable\x12\'\n\x04tags\x18\x08 \x01(\x0b\x32\x19.gauge.messages.ProtoTags\x12\x10\n\x08\x66ileName\x18\t \x01(\t\"j\n\x08ItemType\x12\x08\n\x04Step\x10\x00\x12\x0b\n\x07\x43omment\x10\x01\x12\x0b\n\x07\x43oncept\x10\x02\x12\x0c\n\x08Scenario\x10\x03\x12\x17\n\x13TableDrivenScenario\x10\x04\x12\t\n\x05Table\x10\x05\x12\x08\n\x04Tags\x10\x06\"\xfe\x05\n\rProtoScenario\x12\x17\n\x0fscenarioHeading\x18\x01 \x01(\t\x12\x12\n\x06\x66\x61iled\x18\x02 \x01(\x08\x42\x02\x18\x01\x12+\n\x08\x63ontexts\x18\x03 \x03(\x0b\x32\x19.gauge.messages.ProtoItem\x12\x30\n\rscenarioItems\x18\x04 \x03(\x0b\x32\x19.gauge.messages.ProtoItem\x12\x38\n\x0epreHookFailure\x18\x05 \x01(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x39\n\x0fpostHookFailure\x18\x06 \x01(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x15\n\rexecutionTime\x18\x08 \x01(\x03\x12\x13\n\x07skipped\x18\t \x01(\x08\x42\x02\x18\x01\x12\x12\n\nskipErrors\x18\n \x03(\t\x12\n\n\x02ID\x18\x0b \x01(\t\x12\x30\n\rtearDownSteps\x18\x0c \x03(\x0b\x32\x19.gauge.messages.ProtoItem\x12\"\n\x04span\x18\r \x01(\x0b\x32\x14.gauge.messages.Span\x12\x38\n\x0f\x65xecutionStatus\x18\x0e \x01(\x0e\x32\x1f.gauge.messages.ExecutionStatus\x12\x17\n\x0fpreHookMessages\x18\x0f \x03(\t\x12\x18\n\x10postHookMessages\x18\x10 \x03(\t\x12\x1a\n\x0epreHookMessage\x18\x11 \x03(\tB\x02\x18\x01\x12\x1b\n\x0fpostHookMessage\x18\x12 \x03(\tB\x02\x18\x01\x12\x1e\n\x12preHookScreenshots\x18\x13 \x03(\x0c\x42\x02\x18\x01\x12\x1f\n\x13postHookScreenshots\x18\x14 \x03(\x0c\x42\x02\x18\x01\x12\x1e\n\x16preHookScreenshotFiles\x18\x15 \x03(\t\x12\x1f\n\x17postHookScreenshotFiles\x18\x16 \x03(\t\x12\x14\n\x0cretriesCount\x18\x17 \x01(\x03\"F\n\x04Span\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x03\x12\x11\n\tstartChar\x18\x03 \x01(\x03\x12\x0f\n\x07\x65ndChar\x18\x04 \x01(\x03\"\xa8\x02\n\x18ProtoTableDrivenScenario\x12/\n\x08scenario\x18\x01 \x01(\x0b\x32\x1d.gauge.messages.ProtoScenario\x12\x15\n\rtableRowIndex\x18\x02 \x01(\x05\x12\x1d\n\x15scenarioTableRowIndex\x18\x03 \x01(\x05\x12\x19\n\x11isSpecTableDriven\x18\x04 \x01(\x08\x12\x1d\n\x15isScenarioTableDriven\x18\x05 \x01(\x08\x12\x35\n\x11scenarioDataTable\x18\x06 \x01(\x0b\x32\x1a.gauge.messages.ProtoTable\x12\x34\n\x10scenarioTableRow\x18\x07 \x01(\x0b\x32\x1a.gauge.messages.ProtoTable\"\xdc\x02\n\tProtoStep\x12\x12\n\nactualText\x18\x01 \x01(\t\x12\x12\n\nparsedText\x18\x02 \x01(\t\x12+\n\tfragments\x18\x03 \x03(\x0b\x32\x18.gauge.messages.Fragment\x12\x45\n\x13stepExecutionResult\x18\x04 \x01(\x0b\x32(.gauge.messages.ProtoStepExecutionResult\x12\x17\n\x0fpreHookMessages\x18\x05 \x03(\t\x12\x18\n\x10postHookMessages\x18\x06 \x03(\t\x12\x1e\n\x12preHookScreenshots\x18\x07 \x03(\x0c\x42\x02\x18\x01\x12\x1f\n\x13postHookScreenshots\x18\x08 \x03(\x0c\x42\x02\x18\x01\x12\x1e\n\x16preHookScreenshotFiles\x18\t \x03(\t\x12\x1f\n\x17postHookScreenshotFiles\x18\n \x03(\t\"\xb2\x01\n\x0cProtoConcept\x12.\n\x0b\x63onceptStep\x18\x01 \x01(\x0b\x32\x19.gauge.messages.ProtoStep\x12(\n\x05steps\x18\x02 \x03(\x0b\x32\x19.gauge.messages.ProtoItem\x12H\n\x16\x63onceptExecutionResult\x18\x03 \x01(\x0b\x32(.gauge.messages.ProtoStepExecutionResult\"\x19\n\tProtoTags\x12\x0c\n\x04tags\x18\x01 \x03(\t\"\xac\x01\n\x08\x46ragment\x12;\n\x0c\x66ragmentType\x18\x01 \x01(\x0e\x32%.gauge.messages.Fragment.FragmentType\x12\x0c\n\x04text\x18\x02 \x01(\t\x12,\n\tparameter\x18\x03 \x01(\x0b\x32\x19.gauge.messages.Parameter\"\'\n\x0c\x46ragmentType\x12\x08\n\x04Text\x10\x00\x12\r\n\tParameter\x10\x01\"\xef\x01\n\tParameter\x12>\n\rparameterType\x18\x01 \x01(\x0e\x32\'.gauge.messages.Parameter.ParameterType\x12\r\n\x05value\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12)\n\x05table\x18\x04 \x01(\x0b\x32\x1a.gauge.messages.ProtoTable\"Z\n\rParameterType\x12\n\n\x06Static\x10\x00\x12\x0b\n\x07\x44ynamic\x10\x01\x12\x12\n\x0eSpecial_String\x10\x02\x12\x11\n\rSpecial_Table\x10\x03\x12\t\n\x05Table\x10\x04\"\x1c\n\x0cProtoComment\x12\x0c\n\x04text\x18\x01 \x01(\t\"i\n\nProtoTable\x12.\n\x07headers\x18\x01 \x01(\x0b\x32\x1d.gauge.messages.ProtoTableRow\x12+\n\x04rows\x18\x02 \x03(\x0b\x32\x1d.gauge.messages.ProtoTableRow\"\x1e\n\rProtoTableRow\x12\r\n\x05\x63\x65lls\x18\x01 \x03(\t\"\xf6\x01\n\x18ProtoStepExecutionResult\x12=\n\x0f\x65xecutionResult\x18\x01 \x01(\x0b\x32$.gauge.messages.ProtoExecutionResult\x12\x38\n\x0epreHookFailure\x18\x02 \x01(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x39\n\x0fpostHookFailure\x18\x03 \x01(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x0f\n\x07skipped\x18\x04 \x01(\x08\x12\x15\n\rskippedReason\x18\x05 \x01(\t\"\xa1\x03\n\x14ProtoExecutionResult\x12\x0e\n\x06\x66\x61iled\x18\x01 \x01(\x08\x12\x18\n\x10recoverableError\x18\x02 \x01(\x08\x12\x14\n\x0c\x65rrorMessage\x18\x03 \x01(\t\x12\x12\n\nstackTrace\x18\x04 \x01(\t\x12\x16\n\nscreenShot\x18\x05 \x01(\x0c\x42\x02\x18\x01\x12\x15\n\rexecutionTime\x18\x06 \x01(\x03\x12\x0f\n\x07message\x18\x07 \x03(\t\x12\x41\n\terrorType\x18\x08 \x01(\x0e\x32..gauge.messages.ProtoExecutionResult.ErrorType\x12\x1d\n\x11\x66\x61ilureScreenshot\x18\t \x01(\x0c\x42\x02\x18\x01\x12\x17\n\x0bscreenshots\x18\n \x03(\x0c\x42\x02\x18\x01\x12\x1d\n\x15\x66\x61ilureScreenshotFile\x18\x0b \x01(\t\x12\x17\n\x0fscreenshotFiles\x18\x0c \x03(\t\x12\x14\n\x0cskipScenario\x18\r \x01(\x08\",\n\tErrorType\x12\r\n\tASSERTION\x10\x00\x12\x10\n\x0cVERIFICATION\x10\x01\"\xa9\x01\n\x10ProtoHookFailure\x12\x12\n\nstackTrace\x18\x01 \x01(\t\x12\x14\n\x0c\x65rrorMessage\x18\x02 \x01(\t\x12\x16\n\nscreenShot\x18\x03 \x01(\x0c\x42\x02\x18\x01\x12\x15\n\rtableRowIndex\x18\x04 \x01(\x05\x12\x1d\n\x11\x66\x61ilureScreenshot\x18\x05 \x01(\x0c\x42\x02\x18\x01\x12\x1d\n\x15\x66\x61ilureScreenshotFile\x18\x06 \x01(\t\"\x8b\x05\n\x10ProtoSuiteResult\x12\x34\n\x0bspecResults\x18\x01 \x03(\x0b\x32\x1f.gauge.messages.ProtoSpecResult\x12\x38\n\x0epreHookFailure\x18\x02 \x01(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x39\n\x0fpostHookFailure\x18\x03 \x01(\x0b\x32 .gauge.messages.ProtoHookFailure\x12\x0e\n\x06\x66\x61iled\x18\x04 \x01(\x08\x12\x18\n\x10specsFailedCount\x18\x05 \x01(\x05\x12\x15\n\rexecutionTime\x18\x06 \x01(\x03\x12\x13\n\x0bsuccessRate\x18\x07 \x01(\x02\x12\x13\n\x0b\x65nvironment\x18\x08 \x01(\t\x12\x0c\n\x04tags\x18\t \x01(\t\x12\x13\n\x0bprojectName\x18\n \x01(\t\x12\x11\n\ttimestamp\x18\x0b \x01(\t\x12\x19\n\x11specsSkippedCount\x18\x0c \x01(\x05\x12\x17\n\x0fpreHookMessages\x18\r \x03(\t\x12\x18\n\x10postHookMessages\x18\x0e \x03(\t\x12\x1a\n\x0epreHookMessage\x18\x0f \x03(\tB\x02\x18\x01\x12\x1b\n\x0fpostHookMessage\x18\x10 \x03(\tB\x02\x18\x01\x12\x1e\n\x12preHookScreenshots\x18\x11 \x03(\x0c\x42\x02\x18\x01\x12\x1f\n\x13postHookScreenshots\x18\x12 \x03(\x0c\x42\x02\x18\x01\x12\x0f\n\x07\x63hunked\x18\x13 \x01(\x08\x12\x11\n\tchunkSize\x18\x14 \x01(\x03\x12\x1e\n\x16preHookScreenshotFiles\x18\x15 \x03(\t\x12\x1f\n\x17postHookScreenshotFiles\x18\x16 \x03(\t\"\xbe\x02\n\x0fProtoSpecResult\x12,\n\tprotoSpec\x18\x01 \x01(\x0b\x32\x19.gauge.messages.ProtoSpec\x12\x15\n\rscenarioCount\x18\x02 \x01(\x05\x12\x1b\n\x13scenarioFailedCount\x18\x03 \x01(\x05\x12\x0e\n\x06\x66\x61iled\x18\x04 \x01(\x08\x12\x1b\n\x13\x66\x61iledDataTableRows\x18\x05 \x03(\x05\x12\x15\n\rexecutionTime\x18\x06 \x01(\x03\x12\x0f\n\x07skipped\x18\x07 \x01(\x08\x12\x1c\n\x14scenarioSkippedCount\x18\x08 \x01(\x05\x12\x1c\n\x14skippedDataTableRows\x18\t \x03(\x05\x12%\n\x06\x65rrors\x18\n \x03(\x0b\x32\x15.gauge.messages.Error\x12\x11\n\ttimestamp\x18\x0b \x01(\t\"m\n\x13ProtoScenarioResult\x12,\n\tprotoItem\x18\x01 \x01(\x0b\x32\x19.gauge.messages.ProtoItem\x12\x15\n\rexecutionTime\x18\x02 \x01(\x03\x12\x11\n\ttimestamp\x18\x03 \x01(\t\"i\n\x0fProtoStepResult\x12,\n\tprotoItem\x18\x01 \x01(\x0b\x32\x19.gauge.messages.ProtoItem\x12\x15\n\rexecutionTime\x18\x02 \x01(\x03\x12\x11\n\ttimestamp\x18\x03 \x01(\t\"\xa1\x01\n\x05\x45rror\x12-\n\x04type\x18\x01 \x01(\x0e\x32\x1f.gauge.messages.Error.ErrorType\x12\x10\n\x08\x66ilename\x18\x02 \x01(\t\x12\x12\n\nlineNumber\x18\x03 \x01(\x05\x12\x0f\n\x07message\x18\x04 \x01(\t\"2\n\tErrorType\x12\x0f\n\x0bPARSE_ERROR\x10\x00\x12\x14\n\x10VALIDATION_ERROR\x10\x01\"W\n\x0eProtoStepValue\x12\x11\n\tstepValue\x18\x01 \x01(\t\x12\x1e\n\x16parameterizedStepValue\x18\x02 \x01(\t\x12\x12\n\nparameters\x18\x03 \x03(\t*G\n\x0f\x45xecutionStatus\x12\x0f\n\x0bNOTEXECUTED\x10\x00\x12\n\n\x06PASSED\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\x12\x0b\n\x07SKIPPED\x10\x03\x42\\\n\x16\x63om.thoughtworks.gaugeZ1github.com/getgauge/gauge-proto/go/gauge_messages\xaa\x02\x0eGauge.Messagesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'spec_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.thoughtworks.gaugeZ1github.com/getgauge/gauge-proto/go/gauge_messages\252\002\016Gauge.Messages'
  _globals['_PROTOSPEC'].fields_by_name['preHookMessage']._loaded_options = None
  _globals['_PROTOSPEC'].fields_by_name['preHookMessage']._serialized_options = b'\030\001'
  _globals['_PROTOSPEC'].fields_by_name['postHookMessage']._loaded_options = None
  _globals['_PROTOSPEC'].fields_by_name['postHookMessage']._serialized_options = b'\030\001'
  _globals['_PROTOSPEC'].fields_by_name['preHookScreenshots']._loaded_options = None
  _globals['_PROTOSPEC'].fields_by_name['preHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOSPEC'].fields_by_name['postHookScreenshots']._loaded_options = None
  _globals['_PROTOSPEC'].fields_by_name['postHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOSCENARIO'].fields_by_name['failed']._loaded_options = None
  _globals['_PROTOSCENARIO'].fields_by_name['failed']._serialized_options = b'\030\001'
  _globals['_PROTOSCENARIO'].fields_by_name['skipped']._loaded_options = None
  _globals['_PROTOSCENARIO'].fields_by_name['skipped']._serialized_options = b'\030\001'
  _globals['_PROTOSCENARIO'].fields_by_name['preHookMessage']._loaded_options = None
  _globals['_PROTOSCENARIO'].fields_by_name['preHookMessage']._serialized_options = b'\030\001'
  _globals['_PROTOSCENARIO'].fields_by_name['postHookMessage']._loaded_options = None
  _globals['_PROTOSCENARIO'].fields_by_name['postHookMessage']._serialized_options = b'\030\001'
  _globals['_PROTOSCENARIO'].fields_by_name['preHookScreenshots']._loaded_options = None
  _globals['_PROTOSCENARIO'].fields_by_name['preHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOSCENARIO'].fields_by_name['postHookScreenshots']._loaded_options = None
  _globals['_PROTOSCENARIO'].fields_by_name['postHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOSTEP'].fields_by_name['preHookScreenshots']._loaded_options = None
  _globals['_PROTOSTEP'].fields_by_name['preHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOSTEP'].fields_by_name['postHookScreenshots']._loaded_options = None
  _globals['_PROTOSTEP'].fields_by_name['postHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOEXECUTIONRESULT'].fields_by_name['screenShot']._loaded_options = None
  _globals['_PROTOEXECUTIONRESULT'].fields_by_name['screenShot']._serialized_options = b'\030\001'
  _globals['_PROTOEXECUTIONRESULT'].fields_by_name['failureScreenshot']._loaded_options = None
  _globals['_PROTOEXECUTIONRESULT'].fields_by_name['failureScreenshot']._serialized_options = b'\030\001'
  _globals['_PROTOEXECUTIONRESULT'].fields_by_name['screenshots']._loaded_options = None
  _globals['_PROTOEXECUTIONRESULT'].fields_by_name['screenshots']._serialized_options = b'\030\001'
  _globals['_PROTOHOOKFAILURE'].fields_by_name['screenShot']._loaded_options = None
  _globals['_PROTOHOOKFAILURE'].fields_by_name['screenShot']._serialized_options = b'\030\001'
  _globals['_PROTOHOOKFAILURE'].fields_by_name['failureScreenshot']._loaded_options = None
  _globals['_PROTOHOOKFAILURE'].fields_by_name['failureScreenshot']._serialized_options = b'\030\001'
  _globals['_PROTOSUITERESULT'].fields_by_name['preHookMessage']._loaded_options = None
  _globals['_PROTOSUITERESULT'].fields_by_name['preHookMessage']._serialized_options = b'\030\001'
  _globals['_PROTOSUITERESULT'].fields_by_name['postHookMessage']._loaded_options = None
  _globals['_PROTOSUITERESULT'].fields_by_name['postHookMessage']._serialized_options = b'\030\001'
  _globals['_PROTOSUITERESULT'].fields_by_name['preHookScreenshots']._loaded_options = None
  _globals['_PROTOSUITERESULT'].fields_by_name['preHookScreenshots']._serialized_options = b'\030\001'
  _globals['_PROTOSUITERESULT'].fields_by_name['postHookScreenshots']._loaded_options = None
  _globals['_PROTOSUITERESULT'].fields_by_name['postHookScreenshots']._serialized_options = b'\030\001'
  _globals['_EXECUTIONSTATUS']._serialized_start=5643
  _globals['_EXECUTIONSTATUS']._serialized_end=5714
  _globals['_PROTOSPEC']._serialized_start=31
  _globals['_PROTOSPEC']._serialized_end=536
  _globals['_PROTOITEM']._serialized_start=539
  _globals['_PROTOITEM']._serialized_end=1069
  _globals['_PROTOITEM_ITEMTYPE']._serialized_start=963
  _globals['_PROTOITEM_ITEMTYPE']._serialized_end=1069
  _globals['_PROTOSCENARIO']._serialized_start=1072
  _globals['_PROTOSCENARIO']._serialized_end=1838
  _globals['_SPAN']._serialized_start=1840
  _globals['_SPAN']._serialized_end=1910
  _globals['_PROTOTABLEDRIVENSCENARIO']._serialized_start=1913
  _globals['_PROTOTABLEDRIVENSCENARIO']._serialized_end=2209
  _globals['_PROTOSTEP']._serialized_start=2212
  _globals['_PROTOSTEP']._serialized_end=2560
  _globals['_PROTOCONCEPT']._serialized_start=2563
  _globals['_PROTOCONCEPT']._serialized_end=2741
  _globals['_PROTOTAGS']._serialized_start=2743
  _globals['_PROTOTAGS']._serialized_end=2768
  _globals['_FRAGMENT']._serialized_start=2771
  _globals['_FRAGMENT']._serialized_end=2943
  _globals['_FRAGMENT_FRAGMENTTYPE']._serialized_start=2904
  _globals['_FRAGMENT_FRAGMENTTYPE']._serialized_end=2943
  _globals['_PARAMETER']._serialized_start=2946
  _globals['_PARAMETER']._serialized_end=3185
  _globals['_PARAMETER_PARAMETERTYPE']._serialized_start=3095
  _globals['_PARAMETER_PARAMETERTYPE']._serialized_end=3185
  _globals['_PROTOCOMMENT']._serialized_start=3187
  _globals['_PROTOCOMMENT']._serialized_end=3215
  _globals['_PROTOTABLE']._serialized_start=3217
  _globals['_PROTOTABLE']._serialized_end=3322
  _globals['_PROTOTABLEROW']._serialized_start=3324
  _globals['_PROTOTABLEROW']._serialized_end=3354
  _globals['_PROTOSTEPEXECUTIONRESULT']._serialized_start=3357
  _globals['_PROTOSTEPEXECUTIONRESULT']._serialized_end=3603
  _globals['_PROTOEXECUTIONRESULT']._serialized_start=3606
  _globals['_PROTOEXECUTIONRESULT']._serialized_end=4023
  _globals['_PROTOEXECUTIONRESULT_ERRORTYPE']._serialized_start=3979
  _globals['_PROTOEXECUTIONRESULT_ERRORTYPE']._serialized_end=4023
  _globals['_PROTOHOOKFAILURE']._serialized_start=4026
  _globals['_PROTOHOOKFAILURE']._serialized_end=4195
  _globals['_PROTOSUITERESULT']._serialized_start=4198
  _globals['_PROTOSUITERESULT']._serialized_end=4849
  _globals['_PROTOSPECRESULT']._serialized_start=4852
  _globals['_PROTOSPECRESULT']._serialized_end=5170
  _globals['_PROTOSCENARIORESULT']._serialized_start=5172
  _globals['_PROTOSCENARIORESULT']._serialized_end=5281
  _globals['_PROTOSTEPRESULT']._serialized_start=5283
  _globals['_PROTOSTEPRESULT']._serialized_end=5388
  _globals['_ERROR']._serialized_start=5391
  _globals['_ERROR']._serialized_end=5552
  _globals['_ERROR_ERRORTYPE']._serialized_start=5502
  _globals['_ERROR_ERRORTYPE']._serialized_end=5552
  _globals['_PROTOSTEPVALUE']._serialized_start=5554
  _globals['_PROTOSTEPVALUE']._serialized_end=5641
# @@protoc_insertion_point(module_scope)



class MultipleImplementationFoundException(Exception):
    def __init__(self, message):
        super().__init__(message)
        self.message = message


class SkipScenarioException(Exception):
    def __init__(self, message):
        super().__init__(message)
        self.message = message
